package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import mro.util.*;
import mro.util.properties.*;

import java.util.*;
/*
  import java.util.Vector; 
  import java.util.Map.Entry; 
  import java.util.HashMap; 
  import java.util.HashSet; 
  import java.util.Set; 
*/

/**
 * KMeans
 *      
 * @version          070519
 * @author           Magnus Rosell
 */
public class RandomClusterer extends GroupingAlgorithm{ 

    /*
      protected Properties properties;
      protected UniqueIdentifier identifier;
    */

    private static int randomClusteringNumber = 1;
	


    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new 
	    mro.util.properties.Properties("Random Clustering Properties", CurrentClassGetter.getClassName());
	try{

			
	    Property p = new Property("Number of clusters", 5, "Number of clusers");
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }

    public RandomClusterer(){
	this.properties = getDefaultProperties();
    }	

    public RandomClusterer(mro.util.properties.Properties properties){
	this.properties=properties;
    }
	
	
    public IObjectGrouping cluster(IObjectGroup iogroup){
		
	// Init
	int numOfClusters = (Integer) properties.getPropertyValue("Number of clusters");
	if(numOfClusters > iogroup.numberOfIObjects()){
	    System.out.println("Number of clusters exeeds the number of objects!");
	    System.exit(0);
	}
		
	Vector<IObjectGroup> groups = new Vector<IObjectGroup>(); 
	for(int c=0; c<numOfClusters; c++)
	    groups.add(new IObjectGroup("Cluster "+(c+1)));
	for(int o=0; o<iogroup.numberOfIObjects(); o++){
	    IObject io = iogroup.getIObjectAt(o);
	    int gr = (int) (Math.random() * groups.size());
	    IObjectGroup group = groups.elementAt(gr);
	    group.add(io);
	}

	// Randomize the order.
	for(int c=0; c<groups.size(); c++)
	    groups.elementAt(c).randomOrder();

	String name = "Random Clustering "+randomClusteringNumber;
	randomClusteringNumber++;		
	IObjectGrouping grouping = new IObjectGrouping(name);

	for(int g=0; g<groups.size(); g++){
	    IObjectGroup group = groups.elementAt(g);
	    grouping.add(group);
	    //printGroup(group);
	}
		
	return grouping;
    }

    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}

	return grouping;
    }

	
    public String getInfoString(){
    	return "Random Clusterer";
    }

    public String getAlgorithmName(){
    	return "Random Clusterer";    	
    }
    
    public Vector getPropertyVector(){
    	return properties.getPropertyVector();
    }
    

    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
    	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
    	this.identifier=uniqueIdentifier;
    }

}
