package infomat.algorithms.grouping;

import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.vector.*;

import java.util.*;

import mro.util.CurrentClassGetter;
import mro.util.UniqueIdentifier;
import mro.util.properties.Property;
import mro.util.properties.PropertyNameConflictException;
import mro.util.properties.PropertyNotFoundException;
import mro.util.properties.PropertyValueDescription;

/**
 * The similarity should be the one for the other dimension!
 *
 *
 */
public class RelativeClusterer extends GroupingAlgorithm 
    implements NeedsIMatrix, NeedsIMatrixDimension, NeedsSparseISimilarity{

    private static int relativeClusteringNumber=1;
	
    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new 
	    mro.util.properties.Properties("RelativeClusterer Properties", CurrentClassGetter.getClassName());

	try{


	    Property p = new Property("SimSort", true, "Sort objects in clusters according to similarity to cluster centroid.");
	    PropertyValueDescription pvd = new PropertyValueDescription(true, "Do it.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription(false, "Do not do it.");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;

    }

    private SparseISimilarity similarity;
    private IMatrix matrix;
    private boolean row;

    public RelativeClusterer(){
	this.properties = getDefaultProperties();
    }

    /**
     * The algorithm clusters the opposite dimension specified
     * in the ISimilarity.
     */
    public IObjectGrouping cluster(IObjectGroup iogroup){


	//	IMatrix matrix = similarity.getIMatrix();
	IMatrix matrix = getIMatrix();
	IObjectSet otherSet = matrix.getRowIObjectSet();
	//	boolean row = similarity.getIMatrixDimension();
	boolean row = getIMatrixDimension();
	if(row)
	    otherSet = matrix.getColumnIObjectSet();

	SparseISimilarity otherSimilarity = similarity;
	//	    new SparseDotSimilarity(matrix, !row);

	IObjectGrouping otherGrouping = otherSet.getCurrentGrouping();

	IObjectGrouping relGrouping = 
	    makeWeightGroupingRelativeGroupingOfOtherSet(iogroup, otherGrouping, otherSimilarity);
		
	if((Boolean) properties.getPropertyValue("SimSort"))
	    relGrouping.sort();
		
	String name = "Weight Grouping relative ("+otherGrouping.getName()+") "+relativeClusteringNumber;
	relativeClusteringNumber++;		
	relGrouping.setName(name);

	return relGrouping;

    }

    private IObjectGrouping 
	makeWeightGroupingRelativeGroupingOfOtherSet(IObjectGroup iogroup, IObjectGrouping grouping, ISimilarity similarity){
		
	// Tillverka ny gruppering
	IObjectGrouping coging = 
	    new IObjectGrouping("Weight Grouping relative "+grouping.getName());

	if(similarity instanceof SparseISimilarity){
	    SparseISimilarity sparseSim = (SparseISimilarity) similarity;

	    // Tillverka centroider och nya grupper
	    HashMap<Integer, IObjectGroup> groupingMap = 
		new HashMap<Integer, IObjectGroup>();
	    Vector centroids = new Vector();
	    Vector<IObjectGroup> groups = grouping.getGroups();
	    for(int i=0; i<groups.size(); i++){
		IObjectGroup cog = groups.elementAt(i);
		IVector<IVectorIndex> vec = sparseSim.getGroupRepresentation(cog);
		HashMap<IVectorIndex, Float> centroid = vec.getMap();
		centroids.add(centroid);

		IObjectGroup newCOG = 
		    new IObjectGroup("Weight Group relative "+cog.getString());
		coging.add(newCOG);
		groupingMap.put(i, newCOG);
	    }

	    // G igenom alla objekt och lgg i rtt nya grupp
	    Vector<IObject> vec = iogroup.getOrdered();
	    for(int i=0; i<vec.size(); i++){
		IObject co = vec.elementAt(i);
		int gr=0;
		float grWeight=0.0f;
		for(int g=0; g<centroids.size(); g++){
		    HashMap<IVectorIndex, Float> centroid = 
			(HashMap<IVectorIndex, Float>) centroids.elementAt(g);
		    Float w = centroid.get(co);
		    if(w!=null && w>grWeight){
			gr=g;
			grWeight=w;
		    }
		}
		IObjectGroup cog = groupingMap.get(gr);
		cog.add(co, grWeight);
		coging.add(co, i*10+gr);
	    }
	}else if(similarity instanceof DenseISimilarity){
	    // Not implemented yet.
	    // Don't know if it is interesting.
	}

	return coging;
    }





    /**
     * The algorithm clusters the opposite dimension specified
     * in the ISimilarity.
     */
    public IObjectGrouping cluster(IObjectSet ioset){

	IObjectGrouping grouping = cluster((IObjectGroup) ioset);
	String name = grouping.getName();
	int i=1;
	while(!ioset.setGrouping(grouping)){
	    grouping.setName(name+" ("+i+")");
	    i++;
	}
		
	return grouping;
    }

    /**
     * The similarity should be the one for the other dimension!
     */
    public void setSimilarity(SparseISimilarity similarity){
	this.similarity = similarity;
    }





    public String getInfoString(){
	return "Makes a clustering relative to the grouping of the other dimension.";
    }

    public String getAlgorithmName(){
	return "RelativeClusterer";    	
    }

    public Vector getPropertyVector(){
	return properties.getPropertyVector();
    }


    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }
    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }

    public UniqueIdentifier getUniqueIdentifier(){
	return identifier;
    }

    public void setUniqueIdentifier(UniqueIdentifier uniqueIdentifier){
	this.identifier=uniqueIdentifier;
    }



    public void setIMatrix(IMatrix matrix){
	this.matrix=matrix;
    }

    public IMatrix getIMatrix(){
	return matrix;
    }

    public void setIMatrixDimension(boolean row){
	this.row=row;
    }

    public boolean getIMatrixDimension(){
	return this.row;
    }



}
