package infomat.algorithms.reduction;

import infomat.vectorspace.*;

import java.util.*;


/**
 * Reduces IObjectGroups and IObjectGroupings compared to an other.
 * The class can reduce both in order of the IObject:s in each 
 * IObjectGroup or in random order.
 */
public class SizeReducer implements HasProperties{

    mro.util.properties.Properties properties;

    public SizeReducer(){
	this.properties = getDefaultProperties();
    }

    public SizeReducer(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Size Reducer Properties", CurrentClassGetter.getClassName());
		
	try{

	    Property p = new Property("Reduction Method", 2, "The reduction method");

	    PropertyValueDescription pvd = new PropertyValueDescription("1", "At least \"Number of shared objects\" in common between ILabel and IObject.");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Similarity to the ILabel considered as a centroid. See \"Bigger similarity to ILabel centroid than\".");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Number of shared objects", 1, "The number of objects that has to be shared between the ILabel and the IObjects");
	    properties.addProperty(p);

	    p = new Property("Bigger similarity to ILabel centroid than", 0.0, "IObjects with bigger similarity to the ILabel centroid are saved.");
	    properties.addProperty(p);


	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }


    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public static IObjectGrouping reduceGroupingToSizeOfOtherGroupingInOrder(IObjectGrouping ioging, IObjectGrouping redIOGING, IMatrix matrix){ 
	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> redGroups = redIOGING.getGroups();
	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();
	
	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroupToSizeOfOtherGroupInOrder(groups.elementAt(i), redGroups.elementAt(i));
	    newGroups.add(newGroup);
	}

	
	IObjectSet ioset = matrix.getRowIObjectSet();
	if(!ioset.containsIObjectGrouping(ioging))
	    ioset = matrix.getColumnIObjectSet();
	
	String name = "Order Size Reduced "+ioging.getName();
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);
	int i=1;
	while(!ioset.setGrouping(newIOGING)){
	    newIOGING.setName(name+" ("+i+")");
	    i++;
	}

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * the top IObject:s of "iog". As many as there are in the "redIOG".
     * Currently produces error if the group to reduce is smaller than
     * the group with the proposed size.
     */
    public static IObjectGroup reduceGroupToSizeOfOtherGroupInOrder(IObjectGroup iog, IObjectGroup redIOG){
	int size = redIOG.getOrdered().size();
	Vector<IObject> ordered = 
	    new Vector<IObject>(iog.getOrdered().subList(0, size));
	IObjectGroup newIOG = new IObjectGroup("Order Size Reduced "+iog.getName());
	newIOG.addAll(ordered);
	return newIOG;
    }


    /*************************************/

    /**
     * Note: does not handle missing zero-sized groups!!!
     */
    public static IObjectGrouping reduceGroupingToSizeOfOtherGroupingRandomly(IObjectGrouping ioging, IObjectGrouping redIOGING, IMatrix matrix){ 
	Vector<IObject> ios = ioging.getOrdered();
	Vector<IObjectGroup> groups = ioging.getGroups();
	Vector<IObjectGroup> redGroups = redIOGING.getGroups();
	Vector<IObjectGroup> newGroups = new Vector<IObjectGroup>();

	for(int i=0; i<groups.size(); i++){
	    IObjectGroup newGroup = 
		reduceGroupToSizeOfOtherGroupRandomly(groups.elementAt(i), redGroups.elementAt(i));
	    newGroups.add(newGroup);
	}
	
	IObjectSet ioset = matrix.getRowIObjectSet();
	if(!ioset.containsIObjectGrouping(ioging))
	    ioset = matrix.getColumnIObjectSet();
	
	String name = "Randomly Size Reduced "+ioging.getName();
	IObjectGrouping newIOGING = 
	    new IObjectGrouping(name);
	int i=1;
	while(!ioset.setGrouping(newIOGING)){
	    newIOGING.setName(name+" ("+i+")");
	    i++;
	}

	newIOGING.addAllGroups(newGroups);
	return newIOGING;
    }
    
    /**
     * Makes a new IObjectGroup that consists of 
     * the top IObject:s of "iog". As many as there are in the "redIOG".
     * Currently produces error if the group to reduce is smaller than
     * the group with the proposed size.
     */
    public static IObjectGroup reduceGroupToSizeOfOtherGroupRandomly(IObjectGroup iog, IObjectGroup redIOG){
	int size = redIOG.getOrdered().size();
	Vector<IObject> newVec = new Vector<IObject>();
	Vector<IObject> copyVec = new Vector<IObject>(iog.getOrdered());
	for(int i=0; i<size; i++){
	    int nr = (int) (Math.random() * copyVec.size());
	    newVec.add(copyVec.remove(nr));
	}
	
	IObjectGroup newIOG = new IObjectGroup("Randomly Size Reduced "+iog.getName());
	newIOG.addAll(newVec);
	return newIOG;
    }







}
