package infomat.bridges.jsdm;

import infomat.vectorspace.*;

import mro.util.*;
import mro.util.properties.*;

import moj.ri.*;


import java.io.*;
import java.util.*; 
import java.util.zip.*;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * 
 * @version          080608
 * @author           Magnus Rosell
 */
public class JSDMWrapper implements HasProperties{
    mro.util.properties.Properties properties;

    private SparseDistributedMemory sdm;
    private int numberOfLabels;

    //    private HashMap<IObject, RandomLabel> iorlMap;
    private HashMap<IObject, float[]> ioReprMap;

    private IObjectGroup iogroup;
    private int numberOfIOsNotInSDM;

    public JSDMWrapper(){ 
	this.properties = getDefaultProperties();
	
	//	int dimensionality, int randomDegree, int seed, int leftWindowSize, int rightWindowSize, WeightingScheme weightingScheme);

	int dimensionality = 1800;
	int randomDegree = 8;
	int seed = 710225;
	int leftWindowSize =3;
	int rightWindowSize = 3;
	moj.ri.weighting.WeightingScheme weightingScheme = 
	    new moj.ri.weighting.MangesWS();

	sdm = new SparseDistributedMemory(dimensionality, randomDegree, seed, leftWindowSize, rightWindowSize, weightingScheme);

    }


    public JSDMWrapper(mro.util.properties.Properties properties){
	this.properties = properties;
    }

    public mro.util.properties.Properties getDefaultProperties(){

	mro.util.properties.Properties properties = new mro.util.properties.Properties("Java Sparse Distributed Memory Properties", CurrentClassGetter.getClassName());
		
	try{

	    // Local row weight
	    Property p = new Property("Least Term Frequency", 10, "Minimum Term Frequency to be Imported");
	    properties.addProperty(p);
	    p = new Property("Least Document Frequency", 0, "Minimum Document Frequency to be Imported");
	    properties.addProperty(p);

	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}

	return properties;
    }

    public Object getPropertyValue(String name){
	return properties.getPropertyValue(name);
    }

    public void setProperty(String name, Object value) 
	throws PropertyNotFoundException{
	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
	this.properties=properties;
    }



    /**
     * Loads.
     */
    public void load(String fileName) {
	System.out.println("************** loading "+fileName);

	numberOfLabels = sdm.load(fileName);
	wrap(null);
    }

    /**
     * Loads.
     */
    public void load(String fileName, IObjectGroup group) {
	System.out.println("************** loading "+fileName);

	numberOfLabels = sdm.load(fileName);
	wrap(group);
    }

    private void wrap(IObjectGroup iog){
	Set<Map.Entry<String, RandomLabel>> entries = sdm.entrySet();
	//	iorlMap = new HashMap<IObject, RandomLabel>();
	ioReprMap = new HashMap<IObject, float[]>();
	if(iog==null){
	    iogroup = 
		new IObjectGroup("RI Group "+System.currentTimeMillis());
	    
	    int id = 0;
	    Iterator<Map.Entry<String, RandomLabel>> it = entries.iterator();
	    while(it.hasNext()){
		Map.Entry<String, RandomLabel> entry = it.next();
		IObject io = new IObject(entry.getKey(), id);
		iogroup.add(io);
		//		iorlMap.put(io, entry.getValue());
		float[] repr = entry.getValue().getContext();
		normalize(repr);
		ioReprMap.put(io, repr);
	    }
	}else{
	    iogroup=iog;
	    numberOfIOsNotInSDM=0;
	    Vector<IObject> iovec = iogroup.getOrdered();
	    for(int i=0; i<iovec.size(); i++){
		IObject io = iovec.elementAt(i);
		RandomLabel rl = sdm.getRandomLabel(io.getString());
		if(rl!=null){
		    //		    iorlMap.put(io, rl);
		    float[] repr = rl.getContext();
		    normalize(repr);
		    ioReprMap.put(io, repr);
		}else{
		    numberOfIOsNotInSDM++;
		}
	    }
	}

    }

    private void normalize(float[] repr){
	//	Iterator<float[]> it = ioReprMap.values().iterator();
	double sum = 0.0f;
	for(int i=0; i<repr.length; i++)
	    sum += Math.pow(repr[i], 2);
	for(int i=0; i<repr.length; i++)
	    repr[i] = (float) Math.sqrt(Math.pow(repr[i],2)/sum);
    }


    public float[] getRepresentation(IObject io){
	/*
	RandomLabel rl = iorlMap.get(io);
	return rl.getContext();
	*/
	return ioReprMap.get(io);
    }
    

    public IObjectGroup getWholeIObjectGroup(){
	return iogroup;
    }

}
