package infomat.bridges.jsdm;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;


/**
 * Simple stupic class for clustering RI:s from jsdm.
 *
 * <br>
 * <br>
 * <code>javac -deprecation -classpath ./classes/:lib/sdm.jar -sourcepath ./src ./src/infomat/bridges/jsdm/*.java -d classes</code> 
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/:lib/sdm.jar -Xms3024m -Xmx3024m infomat.bridges.jsdm.RIWrapperClusterer</code>
 *
 * @version          080605
 * @author           Magnus Rosell
 *
 */
public class RIWrapperClusterer{

    public static void main(String[] args) throws IOException, PropertyNotFoundException{


	//	String riFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Code/RI3/testRI";

	String riFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Code/RI3/rodarummet";
	//	String riFileName = "/var/tmp/rosell/KTHNC";
	JSDMWrapper jsdmw = new JSDMWrapper();
	jsdmw.load(riFileName);
	
	IObjectGroup iog = jsdmw.getWholeIObjectGroup();
	

	System.out.println("Lst in");
	System.out.println();

	/*
	String stoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
	Stoplist stoplist = new Stoplist(stoplistFileName);
	boolean stopRows = true;
	stoplist.removeIObjectsInStoplist(matrix, stopRows);

	System.out.println("Stoppat");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();
			 
	Normalizer normalizer = new Normalizer();
	normalizer.normalize(matrix);

	System.out.println("Normaliserat");
	System.out.println("Matrix: "+matrix.getSize());
	System.out.println();
	*/

	/*
	// Load a categorization
	String categorizationFileName = "example/row_grouping.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iobio = new IObjectGroupingIO();
	boolean rows=true; // row categorization
	IObjectGrouping categorization = 
	    iobio.load(rows, matrix, categorizationFile);
	*/


	// Similarity
	JSDMSimilarity jsdmSim = new JSDMSimilarity(jsdmw);

	// Cluster
	/*
	KMeans kmeans = new KMeans(jsdmSim);
	kmeans.setProperty("Number of clusters", 10);
	IObjectGrouping clustering = kmeans.cluster(iog);
	clustering.removeEmptyGroups();
	*/

	BisectingKMeans bkm = new BisectingKMeans(jsdmSim);
	bkm.setProperty("Number of clusters", 10);
	IObjectGrouping clustering = bkm.cluster(iog);
	clustering.removeEmptyGroups();

	System.out.println("Klustrat");


	// The current time
	long time = System.currentTimeMillis();
	
	// Write Text Result
	//	String resultPath = "result/";
	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/riResult/";
	
	String resultFileName = 
	    resultPath+"clusteringResult."+time+".xml";
	File resFile = new File(resultFileName);
	//	GroupingTextIO gtio = new GroupingTextIO();
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	IMatrix matrix = null;
	gtio.save(matrix, clustering, jsdmSim, resFile);

	System.out.println("Sparat resultat textuellt");

	// Evaluate and write the results
	/*
	String evaluationFileName = 
	    resultPath+"clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, jsdmSim, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);
	*/

	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
