package infomat.io;

import infomat.vectorspace.*;
import infomat.vectorspace.vector.*;

import java.io.*;
import java.util.*;

/**
 * For loading and saving a IVector<IVectorString>. 
 * Loads and saves one string at each line.
 * 
 * @version          080805
 * @author           Magnus Rosell
 */
public class IVectorStringIO{

    /**
     * Reads strings from a text file with one string per row.
     */
    public IVector<IVectorString> readFile(String fileName){
	return readFile(new File(fileName));
    }

    public IVector<IVectorString> readFile(File file){
	Vector<String> vec = new Vector<String>();
	IVector<IVectorString> ivec = new IVector<IVectorString>();
	try{
	    BufferedReader bf = new BufferedReader(new FileReader(file));
	    String s = bf.readLine();
	    while(s!=null){
		vec.add(s);
		s = bf.readLine();
	    }
	    bf.close();
	}catch(IOException ioe){
	    System.out.println("Stoplist: Bad IO!");
	}
	int size = vec.size();
	for(int i=0; i<size; i++){
	    ivec.add(new IVectorString(vec.get(i)), (float) size-i);
	}
	return ivec;
    }


    /**
     * Write strings to a text file with one string per row.
     */
    public void writeFile(IVector<IVectorString> ivec, String fileName){
	writeFile(ivec, new File(fileName));
    }

    public void writeFile(IVector<IVectorString> ivec, File file){
	try{
	    PrintWriter out =
		new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));
	    
	    Vector<IVectorString> vec = ivec.getOrderedVector();
	    for(int i=0; i<vec.size(); i++){
		IVectorIndex ii = vec.get(i);
		out.println(ii.getString());
	    }
	    out.close();
	}catch(IOException ioe){
	    System.out.println("Stoplist: Bad IO!");
	}
    }



}
