package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.bridges.tct.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/
// java -cp .:classes/:lib/tct-core.jar:lib/tct-matrix.jar:lib/tct-text.jar -Xms1024m -Xmx1024m infomat.Clusterer -tokenFile "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/jtteLitenKorpus.corpus.tokenFile.xml" -makeIObjectGroupingFromLocation -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/res/"

// java -cp .:classes/:lib/tct-core.jar:lib/tct-matrix.jar:lib/tct-text.jar -Xms1024m -Xmx1024m infomat.Clusterer -tokenFile "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/ATot20.randomPart1.corpus.tokenFile.xml" -makeIObjectGroupingFromLocation -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/tmp/"

// java -cp .:classes/:lib/tct-core.jar:lib/tct-matrix.jar:lib/tct-text.jar -Xms1024m -Xmx1024m infomat.Clusterer -matrix "/afs/nada.kth.se/home/f/u1ya3frf/Code/Infomat/example/matrix.xml" -documentGrouping "/afs/nada.kth.se/home/f/u1ya3frf/Code/Infomat/example/doc_grouping-categorization.xml" -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/tmp/"


// java -cp .:classes/:lib/tct-core.jar:lib/tct-matrix.jar:lib/tct-text.jar -Xms3024m -Xmx3024m infomat.Clusterer -tokenFile "/misc/tcs/language/vol/5/KTHNCTokenFiles/DNTot20.corpus.tokenFil.xml" -makeIObjectGroupingFromLocation -resultPath "/var/tmp/rosell/DN20/"


/**
 *
 * NOT WORKING IN VERSION 080828
 *
 * Class for running clustering tasks without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ -Xms3024m -Xmx3024m infomat.Clusterer</code>
 *
 * @version          080318
 * @author           Magnus Rosell
 *
 */
public class Clusterer implements HasProperties{

    private static String sweStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
    private static String engStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/porter_plus_glasgow_stoplist.txt";

    private Properties properties = null;

    public Clusterer(){
	this.properties = getDefaultProperties();
    }

    public Properties getDefaultProperties(){

	Properties properties = 
	    new Properties("Clusterer Properties", CurrentClassGetter.getClassName());
		
	try{
			
	    /****************/
	    Property p = new Property("In File Type", 0, "What kind of file for input");
	    PropertyValueDescription pvd = new PropertyValueDescription("0", "Use \"Matrix File\"");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Use \"Inpath\" recursively");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Use \"Token File\"");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("3", "Use \"Corpus File\"");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("Matrix File", null, "Matrix File");
	    properties.addProperty(p);
	    p = new Property("Token File", null, "Token File");
	    properties.addProperty(p);
	    p = new Property("Corpus File", null, "Corpus File");
	    properties.addProperty(p);
	    p = new Property("Inpath", null, "Path from where files are read recursively");
	    properties.addProperty(p);

	    /****************/
	    p = new Property("Comparison Grouping", 0, "How to get a comparison grouping");
	    pvd = new PropertyValueDescription("0", "None, or use what's given as main()-argument");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "Use \"IObjectGrouping File\"");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("2", "Construct from IObject location");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("3", "Use \"DocumentGrouping File\"");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    p = new Property("DocumentGrouping File", null, "DocumentGrouping File");
	    properties.addProperty(p);
	    p = new Property("IObjectGrouping File", null, "IObjectGrouping File");
	    properties.addProperty(p);


	    /****************/
	    p = new Property("Language", 0, "What language is used (for stoplist)");
	    pvd = new PropertyValueDescription("0", "Swedish");
	    p.addValueDescription(pvd);
	    pvd = new PropertyValueDescription("1", "English");
	    p.addValueDescription(pvd);
	    properties.addProperty(p);

	    /****************/
	    p = new Property("Result Path", null, "Result Path");
	    properties.addProperty(p);

			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }


    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public mro.util.properties.Properties getProperties(){
    	return properties;
    }

    public void setProperties(mro.util.properties.Properties properties){
    	this.properties=properties;
    }




    public static void main(String[] args) throws IOException, PropertyNotFoundException{
	Clusterer simpleClusterer = new Clusterer();
	simpleClusterer.doIt(args);
    }

    public void doIt(String[] args) throws IOException, PropertyNotFoundException{

	checkInput(args);

	readFiles();

	stoplist();

	System.out.println("Matrix: "+matrix.getSize());
	System.out.println("Categorization: ");
	categorization.printGroupSizes();

	System.out.println("Matrix: "+matrix.getSize());
	filter();
	System.out.println("Matrix: "+matrix.getSize());

	weight();
	System.out.println("Matrix: "+matrix.getSize());

	String resultPath =  (String) getPropertyValue("Result Path");
	cluster(resultPath);

    }


    private void cluster(String resultPath){

	long timeStamp = System.currentTimeMillis();

	IObjectSet set = matrix.getRowIObjectSet();

	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);

	// Cluster
	KMeans kmeans = new KMeans(dotSim);

	IObjectGrouping clustering = kmeans.cluster(set);
	clustering.removeEmptyGroups();

	/*
	try{
	    kmeans.setProperty("Number of clusters", 2);
	}catch(PropertyNotFoundException pnfe){
	    // Do nothing
	    System.out.println("PropertyNotFoundException!");
	}
	TopDownSplitter tds = new TopDownSplitter();
	tds.setSimilarity(dotSim);
	IObjectGrouping clustering = tds.cluster(set, kmeans);
	clustering.removeEmptyGroups();
	*/

	/*
	TCTAgglomerative tctAgg = new TCTAgglomerative();
	IObjectGrouping clustering = tctAgg.cluster(set);
	clustering.removeEmptyGroups();
	*/

	/*
	TCTKMeans tctKMeans = new TCTKMeans();
	IObjectGrouping clustering = tctKMeans.cluster(set);
	clustering.removeEmptyGroups();
	*/

	/*
	Vector<IObjectGroup> vec = clustering.getGroups();
	IObjectGroup group = vec.elementAt(0);

	Vector<Map.Entry<IObject, Float>> centroid = 
	    group.getSortedCentroid(matrix);
	
	for(int i=0; i<centroid.size(); i++){
	    Map.Entry<IObject, Float> entry = centroid.elementAt(i);
	    System.out.println(entry.getKey()+": "+entry.getValue());
	}
	*/


	// Text Result
	String resFileName = resultPath+"clusteringResult."+timeStamp+".xml";
	File resFile = new File(resFileName);
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(matrix, clustering, dotSim, resFile);

	//	PrintWriter out = new PrintWriter(resFile);
	//	gtio.saveGroupingFile(out, clustering, categorization);

	// Evaluation
	Evaluation eval = 
	    new Evaluation(matrix, clustering, categorization, dotSim, ""+timeStamp);
	String evalFile = resultPath+"clusteringEvaluation."+timeStamp+".xml";
	calculateAndSaveEvaluation(eval, evalFile);
	
	System.out.println("Clustered!!!");

    }


    private void calculateAndSaveEvaluation(Evaluation eval, String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



    private void badInput(String comment){
	System.out.println("Bad input!");
	System.out.println(comment);
	System.exit(0);
    }

    
    private void checkInput(String[] args) throws IOException, PropertyNotFoundException{

	if(args.length==0)
	    badInput("To few arguments.");



	for(int i=0; i<args.length; i++){
	    if(args[i].equals("-tokenFile")){
		setProperty("Token File", args[i+1]);
		setProperty("In File Type", 2);
		//		tokenFile=args[i+1];
	    }else if(args[i].equals("-matrix")){
		setProperty("Matrix File", args[i+1]);
		setProperty("In File Type", 0);
		//		matrixFile=args[i+1];
	    }else if(args[i].equals("-corpus")){
		setProperty("Corpus File", args[i+1]);
		setProperty("In File Type", 3);
		//		corpusFile=args[i+1];
	    }else if(args[i].equals("-inpath")){
		setProperty("Inpath", args[i+1]);
		setProperty("In File Type", 1);
		//		inpath=args[i+1];
	    }else if(args[i].equals("-iobjectGrouping")){
		setProperty("Comparison Grouping", 1);
		setProperty("IObjectGrouping File", args[i+1]);
		//		iobjectGroupingFile=args[i+1];
	    }else if(args[i].equals("-documentGrouping")){
		setProperty("Comparison Grouping", 3);
		setProperty("DocumentGrouping File", args[i+1]);
		//		documentGroupingFile=args[i+1];
	    }else if(args[i].equals("-makeIObjectGroupingFromLocation")){
		setProperty("Comparison Grouping", 2);
		//		makeIObjectGroupingFromLocation=true;
	    }else if(args[i].equals("-sve")){
		setProperty("Language", 0);
	    }else if(args[i].equals("-eng")){
		setProperty("Language", 1);
		//		language=1;
	    }else if(args[i].equals("-resultPath")){
		setProperty("Result Path", args[i+1]);
	    }
	}


	String resultPath = (String) getPropertyValue("Result Path");
	System.out.println("----------------------------- "+resultPath);
	File path = new File(resultPath);
	if(!path.exists())
	    path.mkdirs();

	int inFileType = (Integer) getPropertyValue("In File Type");
	String matrixFile = (String) getPropertyValue("Matrix File");
	String tokenFile = (String) getPropertyValue("Token File");
	String corpusFile = (String) getPropertyValue("Corpus File");
	String inpath = (String) getPropertyValue("Inpath");

	int comparisonGrouping = (Integer) 
	    getPropertyValue("Comparison Grouping");
	String documentGroupingFile = (String) 
	    getPropertyValue("DocumentGrouping File");
	String iobjectGroupingFile = (String) 
	    getPropertyValue("IObjectGrouping File");
	boolean makeIObjectGroupingFromLocation =
	    (2==comparisonGrouping);

	int language = (Integer) getPropertyValue("Language");

	if(tokenFile==null && matrixFile==null && 
	    corpusFile==null && inpath==null)
	    badInput("No input file.");

	if(resultPath==null)
	    badInput("No output path");
    }


    private IMatrix matrix = null;
    private IObjectGrouping categorization = null;

    private void readFiles(){

	int inFileType = (Integer) 
	    getPropertyValue("In File Type");
	String matrixFile = (String) 
	    getPropertyValue("Matrix File");
	String inpath =  (String) getPropertyValue("Inpath");
	String tokenFile =  (String) getPropertyValue("Token File");
	String corpusFile =  (String) getPropertyValue("Corpus File");
	
	int comparisonGrouping = (Integer) 
	    getPropertyValue("Comparison Grouping");
	String documentGroupingFile= (String)
	    getPropertyValue("DocumentGrouping File");
	String iobjectGroupingFile = (String) 
	    getPropertyValue("IObjectGrouping File");
	boolean makeIObjectGroupingFromLocation =
	    (2==comparisonGrouping);

	if((iobjectGroupingFile==null && documentGroupingFile==null && 
	    !makeIObjectGroupingFromLocation))
	    badInput("No comparison categorization.");

	// Lsa in matris
	if(tokenFile!=null && inFileType==2){
	    File file = new File(tokenFile);
	    TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	    matrix = tokenIO.load(file);
	}else if(matrixFile!=null && inFileType==0){
	    File file = new File(matrixFile);
	    IMatrixIO iomatrix = new IMatrixIO();
	    matrix = iomatrix.load(file);
	}else if(corpusFile!=null && inFileType==3){
	    File file = new File(corpusFile);
	    CorpusIO ioCorpus = new CorpusIO();
	    matrix = ioCorpus.load(file);
	}else if(inpath!=null && inFileType==1){
	    File file = new File(inpath);
	    TextDirectoryReader tdr = new TextDirectoryReader();
	    matrix = tdr.load(file);
	}else
	    badInput("No input file");

	System.out.println("Opened!!!");

	IObjectGrouping rowGrouping = 
	    matrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    matrix.getRowIObjectSet().makeStartGrouping();
	rowGrouping.sort();
	colGrouping.sort();

	// Lsa in kategorisering
	if(iobjectGroupingFile!=null && comparisonGrouping==1){
	    File file = new File(iobjectGroupingFile);
	    IObjectGroupingIO iobio = new IObjectGroupingIO();
	    boolean row=true; // row categorization
	    categorization = iobio.load(row, matrix, file);
	}else if(documentGroupingFile!=null && comparisonGrouping==3){
	    File file = new File(documentGroupingFile);
	    DocumentGroupingIO dgio = new DocumentGroupingIO(matrix);//this);
	    categorization = dgio.load(file);
	}else if(makeIObjectGroupingFromLocation &&
		 (tokenFile!=null || inpath!=null) && 
		 comparisonGrouping==2){
	    LocationGrouper locationGrouper = new LocationGrouper();
	    categorization = 
		locationGrouper.cluster(matrix.getRowIObjectSet());
	}else
	    badInput("No categorization");
	categorization.sort();
	System.out.print("Categorization: ");
	categorization.printGroupSizes();
	System.out.println("Preprocessed!!!");
    }

    private void stoplist(){

	int language = (Integer) getPropertyValue("Language");

	// Preprocessing
	String stoplistFileName = sweStoplistFileName;
	if(language==1) // engelska
	    stoplistFileName = sweStoplistFileName;
	Stoplist sl = new Stoplist(stoplistFileName);
	boolean row=true;
	sl.removeIObjectsInStoplist(matrix, row);
    }

    private void filter(){
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	System.out.println(matrix + ", " + matrixCellFilter);
	System.out.println("Matrix: "+matrix.getSize());
	matrixCellFilter.filter(matrix);
    }

    private void weight(){
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);
	//	return weighter;
    }



}
