package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/


/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ infomat.mro.JohanPrepare</code>
 * <code>javac -deprecation -classpath ./classes/ -sourcepath ./src ./src/infomat/mro/*.java -d classes</code>
 *
 *
 * @version          080619
 * @author           Magnus Rosell
 *
 */
public class JohanPrepare{

    private static String sweStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
    private static String engStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/porter_plus_glasgow_stoplist.txt";

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	/*
	String corpusFile = "/afs/nada.kth.se/home/f/u1ya3frf/Public/Johan/example/corpus.xml";
	File file = new File(corpusFile);
	CorpusIO ioCorpus = new CorpusIO();
	IMatrix matrix = ioCorpus.load(file);
	matrix.getColumnIObjectSet().makeStartGrouping();
	matrix.getRowIObjectSet().makeStartGrouping();
	*/

	// Load a categorization
	/*
	String documentGroupingFile = "/afs/nada.kth.se/home/f/u1ya3frf/Public/Johan/example/categorization.xml";
	file = new File(documentGroupingFile);
	DocumentGroupingIO dgio = new DocumentGroupingIO(matrix);
	IObjectGrouping categorization = dgio.load(file);
	*/

		// Load the matrix
	// This one has texts as rows and words as columns
	//	String tokenFile = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNCTokenFiles/jtteLitenKorpus.corpus.tokenFile.xml";
	//	String tokenFile = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNCTokenFiles/test.xml";
	//	String tokenFile = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNCTokenFiles/tmp.corpus.tokenFile.xml";
	//	String tokenFile = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNCTokenFiles/ATot20.randomPart1.corpus.tokenFile.xml";
	String tokenFile = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNCTokenFiles/DNTot20.corpus.tokenFile.xml";
	File file = new File(tokenFile);
	TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	IMatrix matrix = tokenIO.load(file);
	matrix.getColumnIObjectSet().makeStartGrouping();
	matrix.getRowIObjectSet().makeStartGrouping();

	// Stoplista
	String stoplistFileName = sweStoplistFileName;
	//	if(language==1) // engelska
	//	    stoplistFileName = engStoplistFileName;
	Stoplist sl = new Stoplist(stoplistFileName);
	boolean stopRow=false;
	sl.removeIObjectsInStoplist(matrix, stopRow);


	// Filter the matrix
	// The example is to small
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(matrix);


	// Weight the matrix
	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(matrix);

	// Cluster the rows (texts) of the matrix
	IObjectSet rowSet = matrix.getRowIObjectSet();

	// Similarity
	boolean row = true; // Similarity between the rows of the matrix.
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);


	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/Johan/";

	String matrixFileName = resultPath + "matrix.xml";
	File matrixFile = new File(matrixFileName);
	IMatrixIO matrixIO = new IMatrixIO();
	matrixIO.save(matrix, matrixFile);
	
	for(int i=0; i<10; i++){
	// Cluster
	KMeans kmeans = new KMeans(dotSim);
	kmeans.setProperty("Repeat number of times", 25);

	IObjectGrouping clustering = kmeans.cluster(rowSet);
	clustering.removeEmptyGroups();
	clustering.sort();

	// The current time
	long time = System.currentTimeMillis();

	// Write clustering as a DocumentGrouping file
	String clusteringFileName = resultPath + "clustering"+time+".xml";
	File clusteringFile = new File(clusteringFileName);
	IObjectGroupingIO iogio = new IObjectGroupingIO();
	iogio.save(clustering, matrix, dotSim, clusteringFile);
	}

	LocationGrouper locationGrouper = new LocationGrouper();
	IObjectGrouping categorization = 
	    locationGrouper.cluster(matrix.getRowIObjectSet());
	String categorizationFileName = resultPath + "categorization.xml";
	File categorizationFile = new File(categorizationFileName);
	IObjectGroupingIO iogio = new IObjectGroupingIO();
	iogio.save(categorization, matrix, categorizationFile);


	
	/*
	// Write Text Result
	String resultFileName = resultPath +
	    "result/clusteringResult."+time+".xml";
	File resFile = new File(resultFileName);
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.save(matrix, clustering, dotSim, resFile);

	// Evaluate and write the results
	String evaluationFileName = resultPath +
	    "result/clusteringEvaluation."+time+".xml";
	Evaluation eval = 
	    new Evaluation(matrix, clustering, categorization, dotSim, ""+time);
	calculateAndSaveEvaluation(eval, evaluationFileName);
	*/

	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }



}
