package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.*;
import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.Iterator;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.xml.sax.SAXException;




/****************/


/**
 * Simple class for running clustering without the GUI.
 *
 * <br>
 * <br>
 * <code>java -cp .:classes/ -Xms3024m -Xmx3024m infomat.WTExperimentator</code>
 *
 * @version          080619
 * @author           Magnus Rosell
 *
 */
public class WTExperimentator{

    private static String sweStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/stoppord/Stoplist.txt";
    private static String engStoplistFileName = "/afs/nada.kth.se/home/f/u1ya3frf/Ordlistor/english/porter_plus_glasgow_stoplist.txt";

    public static void main(String[] args) throws IOException, PropertyNotFoundException{

	// Load the matrix
	//	String tokenFile = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/jtteLitenKorpus.corpus.tokenFile.xml";
	//	String tokenFile = "/afs/nada.kth.se/home/f/u1ya3frf/Korpus/tokenFiles/ATot20.randomPart1.corpus.tokenFile.xml";
	//	String resultPath = "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/WordAndTextExperiment/result/Aftonbladet/";
	String tokenFile = "/afs/nada.kth.se/misc/tcs/language/vol/5/KTHNCTokenFiles/DNTot20.corpus.tokenFil.xml";
	String resultPath = "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/WordAndTextExperiment/result/DN/";
	File file = new File(tokenFile);
	TokenFileTextWordIO tokenIO = new TokenFileTextWordIO();
	IMatrix textMatrix = tokenIO.load(file);
	int language = 0; // swe


	/*
	String resultPath = 
	    "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/WordAndTextExperiment/result/20ng/";
	String inPathName = "/afs/nada.kth.se/misc/tcs/language/vol/5/20newsgroups/processed/20news-bydate-test/";
	File inPathFile = new File(inPathName);
	TextDirectoryReader tdr = new TextDirectoryReader();
	IMatrix textMatrix = tdr.load(inPathFile);
	int language = 1; // eng
	*/

	// Stoplista
	String stoplistFileName = sweStoplistFileName;
	if(language==1) // engelska
	    stoplistFileName = engStoplistFileName;
	Stoplist sl = new Stoplist(stoplistFileName);
	boolean stopRow=false;
	sl.removeIObjectsInStoplist(textMatrix, stopRow);

	IObjectGrouping rowGrouping = 
	    textMatrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    textMatrix.getRowIObjectSet().makeStartGrouping();
	/*
	rowGrouping.sort();
	colGrouping.sort();
	*/

	// Filter the matrix
	// The example is to small
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.filter(textMatrix);

	TFIDFWeighter weighter = new TFIDFWeighter();
	weighter.weight(textMatrix);

	Normalizer textNormalizer = new Normalizer();
	textNormalizer.normalize(textMatrix);
	//	IObjectSet textSet = textMatrix.getRowIObjectSet();
	IObjectGroup allTextsGroup = 
	    new IObjectGroup(textMatrix.getRowIObjectSet());

	LocationGrouper locationGrouper = new LocationGrouper();
	IObjectGrouping textCategorization = 
	    locationGrouper.cluster(textMatrix.getRowIObjectSet());
	
	// Similarity
	boolean row = true; 
	SparseDotSimilarity textDotSim = 
	    new SparseDotSimilarity(textMatrix, row);

	// Orden
	IMatrix wordMatrix = new IMatrix(textMatrix);
	weighter.weight(wordMatrix);
	/*
	Normalizer wordNormalizer = new Normalizer();
	wordNormalizer.setProperty("Normalize", 1); //Columns
	wordNormalizer.normalize(wordMatrix);
	*/

	//	IObjectSet wordSet = wordMatrix.getColumnIObjectSet();
	IObjectGroup allWordsGroup = 
	    new IObjectGroup(textMatrix.getColumnIObjectSet());
	row = false;
	SparseDotSimilarity wordDotSim = 
	    new SparseDotSimilarity(wordMatrix, row);

	IntrinsicGroupEvaluation ige = 
	    new IntrinsicGroupEvaluation(wordDotSim);


	int times = 10;
	int kmeansIterations = 25;

	int[] numberOfTextClusters = {3, 5, 10, 20};
	for(int tc=0; tc<numberOfTextClusters.length; tc++){
	    


	/********** TEXTS ************/

	KMeans textKmeans = new KMeans(textDotSim);
	textKmeans.setProperty("Number of clusters", 
			       numberOfTextClusters[tc]);
	textKmeans.setProperty("Repeat number of times", kmeansIterations);
			       
	String textClusterString = 
	    "0"+numberOfTextClusters[tc]+"textClusters";
	if(numberOfTextClusters[tc]>9)
	    textClusterString = numberOfTextClusters[tc]+"textClusters";

	for(int i=0; i<times; i++){
	    IObjectGrouping textClustering = 
		textKmeans.cluster(allTextsGroup);
	    textClustering.removeEmptyGroups();

	    // The current time
	    long time = System.currentTimeMillis();
	
	    String textEvaluationFileName = 
		resultPath+textClusterString+"/"+"textClusteringEvaluation."+time+".xml";
	    Evaluation textEval = 
		new Evaluation(textMatrix, textClustering, textCategorization, textDotSim, ""+time);
	    calculateAndSaveEvaluation(textEval, textEvaluationFileName);
	}


	/********** ORD & PROJICERING ************/

	// Cluster
	KMeans wordKmeans = new KMeans(wordDotSim);
	wordKmeans.setProperty("Repeat number of times", kmeansIterations);
	
	int[] numberOfWordClusters = {3, 5, 10, 20, 50};
	for(int wc=0; wc<numberOfWordClusters.length; wc++){
	    wordKmeans.setProperty("Number of clusters", 
				   numberOfWordClusters[wc]);
	    
	    for(int i=0; i<times; i++){

		double bestSelfSim = 0.0;
		IObjectGrouping bestWordClustering = null;
		for(int j=0; j<times; j++){
		    IObjectGrouping wordClustering = 
			wordKmeans.cluster(allWordsGroup);
		    wordClustering.removeEmptyGroups();
		    double selfSim = 
			ige.weightedAvgSelfSimilarity(wordClustering);
		    if(j==0){
			bestSelfSim=selfSim;
			bestWordClustering = wordClustering;
		    }else if(selfSim>bestSelfSim){
			bestSelfSim=selfSim;
			bestWordClustering = wordClustering;
		    }
		}

		// Projicera p ordklustren.
		IMatrixProjector mProj = new IMatrixProjector();
		IMatrix projTextMatrix = 
		    mProj.projectAlongColumns(textMatrix, wordDotSim, 
					      bestWordClustering);

		Normalizer projNormalizer = new Normalizer();
		projNormalizer.normalize(projTextMatrix);

		//		IObjectSet projTextSet = projTextMatrix.getRowIObjectSet();
		IObjectGroup allProjTextsGroup = 
		    new IObjectGroup(projTextMatrix.getRowIObjectSet());
		row = true; 
		SparseDotSimilarity projTextDotSim = 
		    new SparseDotSimilarity(projTextMatrix, row);
	
		KMeans projTextKmeans = new KMeans(projTextDotSim);
		projTextKmeans.setProperty("Number of clusters", 
					   numberOfTextClusters[tc]);
		projTextKmeans.setProperty("Repeat number of times", 
					   kmeansIterations);
		IObjectGrouping projTextClustering = 
				     projTextKmeans.cluster(allProjTextsGroup);
		projTextClustering.removeEmptyGroups();

		String wordClusterString = 
		    "0"+numberOfWordClusters[wc]+"wordClusters/";
		if(numberOfWordClusters[wc]>9)
		    wordClusterString = numberOfWordClusters[wc]+"wordClusters/";
		// The current time
		long time = System.currentTimeMillis();
		
		String projTextEvaluationFileName = 
		    resultPath+textClusterString+"-"+wordClusterString+"projTextClusteringEvaluation."+time+".xml";
		Evaluation projTextEval = 
		    new Evaluation(projTextMatrix, projTextClustering, textCategorization, projTextDotSim, ""+time);
		calculateAndSaveEvaluation(projTextEval, projTextEvaluationFileName);

	    }

	}
	}
	System.out.println("Clustered!!!");

    }


    private static void calculateAndSaveEvaluation(Evaluation eval, 
						   String resFileName){
	Measures measures = new Measures("Measures");
	try{
	    measures = eval.calculateAllMeasures();
	}
	catch(MeasureNameConflictException mnce){
	    System.out.println("Measure Name Conflict!");
	}
	measures.save(resFileName);
    }


    private static void printMatrix(IMatrix matrix){
	
	IObjectSet rowSet = matrix.getRowIObjectSet();
	Vector<IObject> rios = rowSet.getOrdered();
	for(int i=0; i<rios.size(); i++){
	    HashSet set = 
		matrix.getColumnIMatrixCellsForRowObject(rios.elementAt(i));
	    Iterator it = set.iterator();
	    while(it.hasNext()){
		IMatrixCell mc = (IMatrixCell) it.next();
		System.out.print(" "+mc.getLocalCount());
	    }
	    System.out.println();
	}

    }


}
