package infomat.mro;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.evaluation.*;
import infomat.io.*;
import infomat.io.textresult.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.vectorspace.weighting.*;

import mro.util.properties.*;
import mro.util.measures.*;

import java.io.*;
import java.util.*;


/**
 * Writes the default Properties of several classes to a directory.
 * Good as a start for setting up experiments. Check that all classes
 * you need are included.
 * <br>
 *
 * @version          080415
 * @author           Magnus Rosell
 *
 *
 *javac -deprecation -classpath ./classes/ -sourcepath ./src ./src/infomat/*.java -d classes
 *
 *
 * java -cp classes/ -Xms1024m -Xmx1024m infomat.WriteProperties -resultPath "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/AllPropertiesWritten/"
 */
public class WriteProperties{
    private static void badInput(){
	System.out.println("Bad input!");
	System.exit(0);
    }

    // "/afs/nada.kth.se/home/f/u1ya3frf/Code/tmp/InfomatRes/"

    public static void main(String args[]) throws IOException{

	if(args.length==0)
	    badInput();
	String resultPath=null;

	for(int i=0; i<args.length; i++){
	    if(args[i].equals("-resultPath"))
		resultPath=args[i+1];
	}
	if(resultPath==null)
	    badInput();

	if(!resultPath.endsWith(File.separator))
	    resultPath += File.separator;

	/**********************************/
	Experimentator exp = new Experimentator();
	exp.getProperties().saveToDirectory(resultPath);
	//	exp.getProperties().save(resultPath+"experimentator.xml");


	/**********************************/
	IMatrix matrix = new IMatrix("Dummy rows", "Dummy cols");
	boolean row = true;
	SparseDotSimilarity dotSim = new SparseDotSimilarity(matrix, row);
	dotSim.getProperties().saveToDirectory(resultPath);

	/**********************************/
	KMeans kmeans = new KMeans(dotSim);
	kmeans.getProperties().saveToDirectory(resultPath);

	BisectingKMeans bisectingKMeans = new BisectingKMeans();
	bisectingKMeans.getProperties().saveToDirectory(resultPath);

	LocationGrouper lg = new LocationGrouper();
	lg.getProperties().saveToDirectory(resultPath);
	//	lg.getProperties().save(resultPath+"locationGrouper.xml");

	RandomClusterer rc = new RandomClusterer();
	rc.getProperties().saveToDirectory(resultPath);
	//	rc.getProperties().save(resultPath+"randomClusterer.xml");

	RelativeClusterer relc = new RelativeClusterer();
	relc.getProperties().saveToDirectory(resultPath);
	//	relc.getProperties().save(resultPath+"relativeClusterer.xml");

	/**********************************/
	IObjectGroupingTextResult gtio = new IObjectGroupingTextResult();
	gtio.getProperties().saveToDirectory(resultPath);
	//	gtio.getProperties().save(resultPath+"groupingTextIO.xml");

	/**********************************/
	IMatrixCellFilter matrixCellFilter = new IMatrixCellFilter();
	matrixCellFilter.getProperties().saveToDirectory(resultPath);
	//	matrixCellFilter.getProperties().save(resultPath+"matrixCellFilter.xml");

	/**********************************/
	TFIDFWeighter tfidfWeighter = new TFIDFWeighter();
	tfidfWeighter.getProperties().saveToDirectory(resultPath);
	//	weighter.getProperties().save(resultPath+"weighter.xml");

	/**********************************/
	LabelReducer lbred = new LabelReducer();
	lbred.getProperties().saveToDirectory(resultPath);
	//	lbred.getProperties().save(resultPath+"labelReducer.xml");

	CentroidLabeler centroidLabeler = new CentroidLabeler();
	centroidLabeler.getProperties().saveToDirectory(resultPath);

	RelativeLabeler relativeLabeler = new RelativeLabeler(matrix, row);
	relativeLabeler.getProperties().saveToDirectory(resultPath);

	WeightRankingBoostLabeler weightRankingBoostLabeler = 
	    new WeightRankingBoostLabeler(tfidfWeighter, matrix, row);
	weightRankingBoostLabeler.getProperties().saveToDirectory(resultPath);
	//	weightRankingBoostLabeler.getProperties().save(resultPath+"weightRankingBoostLabeler.xml");

	CentroidRankingBoostLabeler centroidRankingBoostLabeler = 
	    new CentroidRankingBoostLabeler();
	centroidRankingBoostLabeler.getProperties().saveToDirectory(resultPath);

	CentroidBoostLabeler centroidBoostLabeler = 
	    new CentroidBoostLabeler();
	centroidBoostLabeler.getProperties().saveToDirectory(resultPath);

	CombinedCentroidLabeler combinedCentroidLabeler = 
	    new CombinedCentroidLabeler();
	combinedCentroidLabeler.getProperties().saveToDirectory(resultPath);

	//	centroidRankingBoostLabeler.getProperties().save(resultPath+"centroidRankingBoostLabeler.xml");

	/*
	SizeReducer sizered = new SizeReducer();
	sizered.getProperties().save(resultPath+"sizeReducer.xml");
	*/

    }
}
