package infomat.vectorspace;

import infomat.evaluation.measures.Measure;
import infomat.evaluation.measures.Measures;

import java.util.*;


/**
 * IObjectGroupCalculator helps with calculations on IObjectGroup:s.
 *
 * @version          080128
 * @author           Magnus Rosell
 */
public class IObjectGroupCalculator{
    
    private IObjectGroup group;
    private IMatrix matrix;
    private int totalLocalCount;

    public IObjectGroupCalculator(IObjectGroup group, IMatrix matrix){
	this.group=group;
	this.matrix=matrix;
    }




    public HashMap<IObject, Float> localRelativeFrequency(){
	HashMap<IObject, Float> objectRelativeFrequencyMap = 
	    new HashMap<IObject, Float>();
	HashMap<IObject, Integer> objectCountMap=localCount();
	Iterator<IObject> it = objectCountMap.keySet().iterator();
	while(it.hasNext()){
	    IObject io = it.next();
	    float relFreq = ((float) objectCountMap.get(io))/totalLocalCount;
	    objectRelativeFrequencyMap.put(io, relFreq);
	}
	return objectRelativeFrequencyMap;
    }


    public HashMap<IObject, Integer> localCount(){
	totalLocalCount = 0;
	HashMap<IObject, Integer> objectCountMap = 
	    new HashMap<IObject, Integer>();
	Vector<IObject> ios1 = group.getOrdered();
	for(int i=0; i<ios1.size(); i++){
	    IObject io1 = ios1.elementAt(i);
	    Vector cells = matrix.getCellsWith(io1);
	    for(int j=0; j<cells.size(); j++){
		IMatrixCell mc = (IMatrixCell) cells.elementAt(j);
		IObject io2 = mc.getOtherIObject(io1);
		Integer count = objectCountMap.get(io2);
		if(count==null)
		    objectCountMap.put(io2, mc.getLocalCount());
		else
		    objectCountMap.put(io2, count + mc.getLocalCount());
		totalLocalCount += mc.getLocalCount();
	    }
	}
	return objectCountMap;
    }


}

