package infomat.view;

import infomat.algorithms.grouping.*;
import infomat.algorithms.labeling.*;
import infomat.bridges.jsdm.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.algorithms.*;
import infomat.view.tools.*;
import infomat.view.grouping.settings.*;
import infomat.view.pixel.*;
import infomat.view.sorter.*;
import infomat.view.similarity.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


/**
 * The heart! Keeps track of what the corpus and other stuff contains.
 */
public class InfomatModel extends Observable
    implements HasProperties{

    private IMatrix matrix;
	
    private MainWindow mainWindow;
    private InfomatView view;

    /*
      private final static Color[] farger = { Color.BLUE, Color.CYAN,
      Color.MAGENTA, Color.GREEN, 
      Color.YELLOW, Color.PINK };
    */
	
    private mro.util.properties.Properties properties;
    private static String defaultSettingsFile = "InfomatProperties.xml";
    private String settingsFile;
	
    private ProgressFrame progressFrame;
	
    public InfomatModel(MainWindow mainWindow){
	this.mainWindow=mainWindow;
	String sep = System.getProperty("file.separator");
	settingsFile = System.getProperty("user.dir")+sep+"settings"+sep+defaultSettingsFile;
	//System.out.println("settingsFile: "+settingsFile);
	properties = getDefaultProperties();

	similarityFrame = new SimilarityFrame(this);
	toggleSimilarityFrame();

	/*
	  mro.util.properties.Property p = new 
	  mro.util.properties.Property("User dir", System.getProperty("user.dir"), "User directory");
	  try{
	  properties.addProperty(p);
	  }catch(PropertyNameConflictException pnce){
	  System.out.println("Bad property name"+p.getName());
	  System.out.println("Exiting!");
	  System.exit(0);
	  }
	*/
	//System.out.println("----------"+properties.getPropertyValue("Icon Path"));
	//System.out.println("----------"+properties.getPropertyValue("Icon Path"));
    }
	
    public mro.util.properties.Properties getDefaultProperties(){
	mro.util.properties.Properties p = null;
	try{
	    p = mro.util.properties.Properties.load(settingsFile);
	}catch(Exception e){
	    System.out.println(e);
	    e.printStackTrace();
	    p = getFallbackProperties();
	}
	return p;
    }


    public mro.util.properties.Properties getFallbackProperties(){

	mro.util.properties.Properties properties = 
	    new mro.util.properties.Properties("Infomat Properties", mro.util.CurrentClassGetter.getClassName());
		
	try{
			
	    Property p = new Property("Matrix File Path", "example/", "Location of Matrix Files.");
	    properties.addProperty(p);

	    p = new Property("Result Path", "result/", "Where results are written.");
	    properties.addProperty(p);

	    p = new Property("Icon Path", "icons/", "Location of menu icons.");
	    properties.addProperty(p);

	    /*
	    p = new Property("Browser", "firefox", "Browser to open objects (locations).");
	    properties.addProperty(p);
	    */
			
	}catch(PropertyNameConflictException pnce){
	    // Won't happen here
	}
	return properties;
    }





    public Object getPropertyValue(String name){
    	return properties.getPropertyValue(name);
    }
    public void setProperty(String name, Object value) throws PropertyNotFoundException{
    	properties.setProperty(name, value);
    }

    public Properties getProperties(){
    	return properties;
    }
    
    public void setProperties(Properties properties){
    	this.properties=properties;
    }
	
    public void setChanged(){
	super.setChanged();
    }
	
	
	
    /**
     * Do we have a matrix?
     * 
     * @return true if we have a matrix
     */
    public boolean hasIMatrix() {
	return matrix != null;
    }

    /**
     * 
     * @return the current matrix object
     */
    public IMatrix getIMatrix() {
	return matrix;
    }

    /**
     * Resets the matrix, i.e. sets it to null; no matrix.
     */
    public void resetIMatrix() {
	matrix=null;
    }

    public MainWindow getMainWindow(){
	return mainWindow;
    }


    public ISimilarity getISimilarity(boolean row){
	if(similarityFrame==null)
	    return null;
	return similarityFrame.getISimilarity(row);
	//	return new SparseDotSimilarity(matrix, row);
    }

    public SimilarityManager getSimilarityManager(){
	return similarityFrame.getSimilarityManager();
    }
	
    public IObjectGrouping getCurrentGrouping(boolean row){
	if(row)
	    return matrix.getCurrentRowGrouping();
	return matrix.getCurrentColumnGrouping();
    }

    /**
     * New matrix.
     */
    /*
      public void newIMatrix(){
      matrix = new IMatrix();
      }
    */

    /*
      public Color[] getFarger(){
      return farger;
      }
    */

    /**
     * Set the view. Must me done!!
     * 
     * @param view
     *            The view.
     */
    public void setView(InfomatView view) {
	this.view = view;
    }


    public void update(){
	setChanged();
	notifyObservers();
    }


    /**
     * Opens a Token File to read Texts and Words.
     */
    public void openTokenFileForTextWord(File file){
		
	progressFrame = new ProgressFrame("Loading Corpus File");
	matrix = null;
	view.forceChange();
	setChanged();
	notifyObservers();
		
	TokenFileTextWordIO tfio = new TokenFileTextWordIO();
	matrix = tfio.load(file);

	// Make start groupings
	//IObjectGrouping colGrouping = 
	matrix.getColumnIObjectSet().makeStartGrouping();
	//IObjectGrouping rowGrouping = 
	matrix.getRowIObjectSet().makeStartGrouping();

	progressFrame.dispose();

	// Grouping Settings
	boolean row=true;
	view.setRowGroupingManager(new GroupingSettingsManager(view, this, row));
	row=false;
	view.setColumnGroupingManager(new GroupingSettingsManager(view, this, row));
		
	view.forceChange();
	//view.getOverview().forceChange();
	setChanged();
	notifyObservers();
    }




    /**
     * Opens a Corpus File.
     */
    public void openCorpusFile(File file){
		
	progressFrame = new ProgressFrame("Loading Corpus File");
	matrix = null;
	view.forceChange();
	setChanged();
	notifyObservers();
		
	CorpusIO cio = new CorpusIO();
	matrix = cio.load(file);

	// Make start groupings
	//IObjectGrouping colGrouping = 
	matrix.getColumnIObjectSet().makeStartGrouping();
	//IObjectGrouping rowGrouping = 
	matrix.getRowIObjectSet().makeStartGrouping();

	progressFrame.dispose();

	// Grouping Settings
	boolean row=true;
	view.setRowGroupingManager(new GroupingSettingsManager(view, this, row));
	row=false;
	view.setColumnGroupingManager(new GroupingSettingsManager(view, this, row));
		
	view.forceChange();
	//view.getOverview().forceChange();
	setChanged();
	notifyObservers();
    }

    /*
      private IObjectGrouping makeRelativeColumnGrouping(IObjectGrouping rowGrouping){
		
      // Make Column Grouping relative Row Grouping
      IObjectSet cols = matrix.getColumnIObjectSet();
      IObjectGrouping relativeColGrouping = 
      Sorter.makeWeightGroupingRelativeGroupingOfOtherSet(cols, rowGrouping, matrix);
      // Unikt namn
      String name=relativeColGrouping.getName();
      int i=1;
      while(!cols.setGrouping(relativeColGrouping)){
      relativeColGrouping.setName(name+" ("+i+")");
      i++;
      }
      return relativeColGrouping;
      }
    */

    /**
     * Opens a Document Grouping File.
     */
    public void openDocumentGroupingFile(File file){
	DocumentGroupingIO dgio = new DocumentGroupingIO(matrix);
	IObjectGrouping rowGrouping = dgio.load(file);
	rowGrouping.sort();
	IObjectSet cos = matrix.getRowIObjectSet();
	System.out.println("Lst in!!!");

	//fundamentalIMatrixChanges=true;

	// Just nu stts grupperingen till den man senast lst in!
	String name = rowGrouping.getName();
	int i=1;
	while(!cos.setGrouping(rowGrouping)){
	    rowGrouping.setName(name+" ("+i+")");
	    i++;
	}
	//	cos.addGrouping(rowGrouping);



	// Grouping Settings
	view.getRowGroupingManager().putNewGrouping(rowGrouping);

	/*
	  IObjectGrouping relativeColGrouping = 
	  makeRelativeColumnGrouping(rowGrouping);
	  view.getColumnGroupingManager().putNewGrouping(relativeColGrouping);
	*/

	view.forceChange();
	setChanged();
	notifyObservers();
    }

    /*
      public boolean inquireFundamentalChanges(){
      boolean ret = fundamentalIMatrixChanges;
      fundamentalIMatrixChanges=false;
      return ret;
      }
    */


    public void removeColumns(int colFrom, int colTo){
	//matrix.removeColumns(colFrom, colTo);
	matrix.getCurrentColumnGrouping().remove(colFrom, colTo);
    }


    public void removeRows(int rowFrom, int rowTo){
	//matrix.removeRows(colFrom, colTo);
	matrix.getCurrentRowGrouping().remove(rowFrom, rowTo);
    }


    public void resortIMatrix(){
	matrix.resort();
	update();
    }


    /**
     * Reset settings about zoom and position in the image.
     * 
     */
    public void resetImage() {
	view.resetImage();
	update();
    }


    public void saveIMatrixPicture(File file){
	view.saveIMatrixPicture(file);
    }

    public void saveIMatrixFile(File file){
	matrix.saveIMatrix(file);
    }

    public void saveIObjectGroupingFile(IObjectGrouping grouping, File file){
	matrix.saveIObjectGroupingFile(grouping, file);
    }



    public void openRIFile(File file){
	matrix = null;
	view.forceChange();
	setChanged();
	notifyObservers();
		
	RIIO riio = new RIIO();
	matrix = riio.load(file);
	
	Normalizer normalizer = new Normalizer();
	normalizer.normalize(matrix);

	//	System.out.println("---------------- "+matrix);

	IObjectGrouping rowGrouping = 
	    matrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    matrix.getRowIObjectSet().makeStartGrouping();

	rowGrouping.sort();
	colGrouping.sort();

	// Grouping Settings
	boolean row=true;
	view.setRowGroupingManager(new GroupingSettingsManager(view, this, row));
	row=false;
	view.setColumnGroupingManager(new GroupingSettingsManager(view, this, row));

	/*
	// Colorings
	view.getRowGroupingManager().makeNewColoringsForAllGroupings();
	view.getColumnGroupingManager().makeNewColoringsForAllGroupings();
	*/
		
	/*
	  boolean row=true;
	  view.getColoringManager().putNewColoringsForAllGroupings(row, matrix.getRowIObjectSet());
	  row=false;
	  view.getColoringManager().putNewColoringsForAllGroupings(row, matrix.getColumnIObjectSet());
	*/

	setChanged();
	notifyObservers();
    }



	
    public void openIMatrixFile(File file){

	//	System.out.println("************** "+file.getPath());
		
	matrix = null;
	view.forceChange();
	setChanged();
	notifyObservers();
		
	IMatrixIO iomatrix = new IMatrixIO();
	matrix = iomatrix.load(file);

	//	System.out.println("---------------- "+matrix);

	IObjectGrouping rowGrouping = 
	    matrix.getColumnIObjectSet().makeStartGrouping();
	IObjectGrouping colGrouping = 
	    matrix.getRowIObjectSet().makeStartGrouping();

	rowGrouping.sort();
	colGrouping.sort();

	// Grouping Settings
	boolean row=true;
	view.setRowGroupingManager(new GroupingSettingsManager(view, this, row));
	row=false;
	view.setColumnGroupingManager(new GroupingSettingsManager(view, this, row));

	/*
	// Colorings
	view.getRowGroupingManager().makeNewColoringsForAllGroupings();
	view.getColumnGroupingManager().makeNewColoringsForAllGroupings();
	*/
		
	/*
	  boolean row=true;
	  view.getColoringManager().putNewColoringsForAllGroupings(row, matrix.getRowIObjectSet());
	  row=false;
	  view.getColoringManager().putNewColoringsForAllGroupings(row, matrix.getColumnIObjectSet());
	*/

	setChanged();
	notifyObservers();
    }

    public IObjectGrouping openIObjectGroupingFile(boolean row, File file){
	IObjectGroupingIO iogrouping = new IObjectGroupingIO(); 
	IObjectGrouping grouping = iogrouping.load(row, matrix, file);

	if(grouping!=null){
	    IObjectSet set = matrix.getColumnIObjectSet();
	    if(row)
		set = matrix.getRowIObjectSet();

	    String name = grouping.getName();
	    int i=1;
	    while(!set.addGrouping(grouping)){
		grouping.setName(name+" ("+i+")");
		i++;
	    }

	    view.getGroupingManager(row).putNewGrouping(grouping);
	}

	return grouping;
    }
	
	
	
    public void purgeMatrix(){
	getIMatrix().purgeMatrixToCurrentGroupings();
	setChanged();
	notifyObservers();
    }

    private MatrixFilterFrame matrixFilterFrame;
    public void filterMatrix(){
	if(matrixFilterFrame!=null)
	    matrixFilterFrame.dispose();
	matrixFilterFrame = new MatrixFilterFrame(matrix, this, view);
    }

    private MatrixWeightingFrame matrixWeightingFrame;
    public void weightMatrix(){
	if(matrixWeightingFrame!=null)
	    matrixWeightingFrame.dispose();
	matrixWeightingFrame = new MatrixWeightingFrame(matrix, this, view);
    }

    public void invertMatrix(){
	/*
	  System.out.println("model.invertMatrix()");
	  System.out.println("current matrix size: "+getIMatrix().getCurrentSize());
	  System.out.println("current col grouping: "+getIMatrix().getCurrentColumnGrouping().getName());
	  System.out.println("current row grouping: "+getIMatrix().getCurrentRowGrouping().getName());
	  System.out.println("current row manager: ");
	  view.getRowGroupingManager().print();
	  System.out.println("current col manager: ");
	  view.getColumnGroupingManager().print();
	*/
	// Invert Matrix
	getIMatrix().invertMatrix();

	// Invert View 
	view.invert();
		
	similarityFrame.invert();

	/*
	  System.out.println("-------------");
	  System.out.println("current matrix size: "+getIMatrix().getCurrentSize());
	  System.out.println("current col grouping: "+getIMatrix().getCurrentColumnGrouping().getName());
	  System.out.println("current row grouping: "+getIMatrix().getCurrentRowGrouping().getName());
	  System.out.println("current row manager: ");
	  view.getRowGroupingManager().print();
	  System.out.println("current col manager: ");
	  view.getColumnGroupingManager().print();
	*/
		
	//view.changePartPictures();
	//view.getOverview().changePartPictures();
		
	//view.getSparsePicture().makeImage();
	//view.getOverview().makeImage();
		
	view.forceChange();
	resetImage();
    }

	
    private LabelFrame labelFrame;
    public void labeler(){
	if(labelFrame!=null)
	    labelFrame.dispose();
	labelFrame = new LabelFrame(matrix, this, view);	
	//	Labeler.testLabeler(getIMatrix().getCurrentRowGrouping(), getIMatrix());
    }


    // One can have as many EvaluationFrames open as one wishes.
    //	Should there be an upper limit or something??
    private EvaluationFrame evaluationFrame; 
    public void evaluation(){
	//		if(evaluationFrame!=null)
	//			evaluationFrame.dispose();
	evaluationFrame = new EvaluationFrame(matrix, this, view);

    }
    //	 One can have as many SorterFrames open as one wishes.
    //	Should there be an upper limit or something??
    private SorterFrame sorterFrame; 
    public void sorter(){
	//		if(sorterFrame!=null)
	//			sorterFrame.dispose();
	sorterFrame = new SorterFrame(matrix, this, view);

    }
	
    private AlgorithmFrame algorithmFrame = null;
    public void cluster(){
	if(matrix==null){
	    if(algorithmFrame!=null){
		algorithmFrame.dispose();
		algorithmFrame=null;
	    }
	    return;
	}
	if(algorithmFrame==null)
	    algorithmFrame = new AlgorithmFrame(this, view);
	algorithmFrame.setVisible(true);
    }

    private MatrixSummaryFrame matrixSummaryFrame;
    public void matrixSummary(){
	if(matrixSummaryFrame!=null)
	    matrixSummaryFrame.dispose();
	matrixSummaryFrame = new MatrixSummaryFrame(matrix, this, view);	
    }

    private ExportTextFrame exportTextFrame;
    public void exportText(){
	if(exportTextFrame!=null)
	    exportTextFrame.dispose();
	exportTextFrame = new ExportTextFrame(matrix, this, mainWindow);	
    }
	
    private StoplistFrame stoplistFrame;
    public void stoplist(){
	if(stoplistFrame!=null)
	    stoplistFrame.dispose();
	stoplistFrame = new StoplistFrame(matrix, this, view);	
    }

    private InfomatPropertiesFrame propertiesFrame;
    public void showProperties(){
	if(propertiesFrame!=null)
	    propertiesFrame.dispose();
	propertiesFrame = new InfomatPropertiesFrame(this);
    }

    public void alteredProperties(){
	mainWindow.getSessionPanel().getToolbarView().putAll();
	mainWindow.getSessionPanel().putAll();
    }

    public void toggleGuide() {
	view.toggleGuide();
    }

    private PixelFrame pixelFrame;
    public void togglePixelFrame(){
	if(matrix==null){
	    if(pixelFrame!=null){
		pixelFrame.dispose();
		pixelFrame=null;
	    }
	    return;
	}
	if(pixelFrame==null)
	    pixelFrame = new PixelFrame(this, view);	
	else
	    pixelFrame.toggleVisible();
    }

    /*
    private PixelFrame pixelFrame;
    public void pixelFrame(){
	if(matrix==null)
	    return;
	if(pixelFrame!=null)
	    pixelFrame.dispose();
	pixelFrame = new PixelFrame(this, view);	
    }
    */

    public PixelFrame getPixelFrame(){
	return pixelFrame;
    }


    private SimilarityFrame similarityFrame;
    public void toggleSimilarityFrame(){
	similarityFrame.toggleVisible();
    }



}
