package infomat.view.similarity;

import infomat.algorithms.grouping.*;
import infomat.io.*;
import infomat.vectorspace.*;
import infomat.vectorspace.similarity.*;
import infomat.view.*;
import infomat.view.util.*;
import infomat.view.grouping.settings.*;
import mro.util.properties.*;
import mro.util.properties.Properties;
import mro.util.properties.dialog.*;

import java.awt.*;
import java.awt.Dialog.*;//ModalityType;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;
import java.io.*;

import javax.swing.*;


	
/**
 *
 * @version          080811
 * @author           Magnus Rosell
 */
public class SimilarityPanel extends JPanel
    implements ActionListener{

    private InfomatModel model;
    private boolean row;
    //    private SimilaritySelectionPanel similaritySelectionPanel;
    private SimilarityManager similarityManager;

    private PropertiesPanel propertiesPanel;

    private JComboBox cBox;
    private String selected;
    private Vector<String> simNames = null;
    private HashMap<String, SparseISimilarity> nameSimMap = null;



    public SimilarityPanel(SimilarityManager similarityManager, 
			   InfomatModel model,
			   boolean row){
	this.similarityManager=similarityManager;
	this.model=model;
	this.row=row;

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	/*
	similaritySelectionPanel = 
	    new SimilaritySelectionPanel(similarityManager, this, model, row);
	*/


	initSimilarities();
	resetSimilarities();


	putAll();
    }





    public void putAll(){
	removeAll();
	    
	if(model.getIMatrix()!=null){
	    //	    add(similaritySelectionPanel);

	    resetSimilarities();
	    add(cBox);


	    String path = (String) model.getPropertyValue("Result Path");
	    propertiesPanel = 
		new PropertiesPanel(getSelectedSimilarity(), path);
	    add(propertiesPanel);
	}

	revalidate();
	repaint();
    }



	
    private void initSimilarities(){
	simNames = new Vector<String>();
	nameSimMap = new HashMap<String, SparseISimilarity>();
	
	SparseISimilarity sim = 
	    new SparseDotSimilarity(model.getIMatrix(), row);
	nameSimMap.put(sim.getName(), sim);
	simNames.add(sim.getName());
	

	sim = new SparseOverlapSimilarity(model.getIMatrix(), row);
	nameSimMap.put(sim.getName(), sim);
	simNames.add(sim.getName());

	/*
	sim = new SparseLiteralSimilarity();
	nameSimMap.put(sim.getName(), sim);
	simNames.add(sim.getName());
	*/

	selected = simNames.get(0);
	cBox = new JComboBox(simNames);
	cBox.setSelectedItem(selected);
	add(cBox);
	cBox.addActionListener(this);
    }

    protected void resetSimilarities(){
	/*
	simNames = new Vector<String>();
	nameSimMap = new HashMap<String, SparseISimilarity>();
	
	SparseISimilarity sim = 
	    new SparseDotSimilarity(model.getIMatrix(), row);
	nameSimMap.put(sim.getName(), sim);
	simNames.add(sim.getName());
	
	sim = new SparseLiteralSimilarity();
	nameSimMap.put(sim.getName(), sim);
	simNames.add(sim.getName());

	cBox = new JComboBox(simNames);
	cBox.setSelectedItem(selected);
	add(cBox);
	cBox.addActionListener(this);

	similarityManager.setSimilarity(getSelectedSimilarity(), row);
	*/

	for(int i=0; i<simNames.size(); i++){
	    String name = simNames.get(i);
	    SparseISimilarity sim = nameSimMap.get(name);
	    if(sim instanceof NeedsIMatrix)
		((NeedsIMatrix) sim).setIMatrix(model.getIMatrix());
	    if(sim instanceof NeedsIMatrixDimension)
		((NeedsIMatrixDimension) sim).setIMatrixDimension(row);
	}

	similarityManager.setSimilarity(getSelectedSimilarity(), row);
    }

    private boolean actionLeadToChange(){
	String newSelected = (String) cBox.getSelectedItem();
	//	if(selected != newSelected){
	if(!selected.equals(newSelected)){
	    selected = newSelected;
	    return true;
	}
	return false;
    }

    protected SparseISimilarity getSelectedSimilarity(){
	return nameSimMap.get(selected);
    }


    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JComboBox){ 
	    /*
	    if(actionLeadToChange()){
		putAll();
	    }
	    */
	    actionLeadToChange();
	    putAll();
	}


    }





}
