package mro.util;

import java.io.*; 
import java.util.*; 

/**
 * Converts filenames to a format using the current file separator.
 *
 * @version          080827
 * @author           Magnus Rosell
 */
public class PathHelper{


    private static String currentSeparator = File.separator;
    private static String separators = "\\\\/";

    public static String convertToCurrentFileSeparator(String fileName){
	if(fileName==null)
	    return null;
	String[] parts = fileName.split("["+separators+"]");
	String ret = parts[0];
	if(beginsWithSeparator(fileName))
	    ret = currentSeparator + ret;
	for(int i=1; i<parts.length; i++)
	    ret = ret + currentSeparator + parts[i];
	if(endsWithSeparator(fileName))
	    ret = ret + currentSeparator;
	return ret;
    }


    public static boolean beginsWithSeparator(String fileName){
	return separatorAtPos(fileName, 0);
    }

    public static boolean endsWithSeparator(String fileName){
	return separatorAtPos(fileName, fileName.length()-1);
    }

    private static boolean separatorAtPos(String fileName, int pos){
	if(fileName == null || pos >= fileName.length())
	    return false;
	for(int i=0; i<separators.length(); i++){
	    if(fileName.charAt(pos)==separators.charAt(i))
		return true;
	}
	return false;
    }

    public static String[] splitFileNameAtSeparators(String fileName){
        return fileName.split("["+separators+"]");
    }


    /**
     * Used by infomat.algorithms.LocationGrouper
     */
    public static String longestCommonStart(String s1, String s2){
	String ret = "";
	String[] s1s = s1.split("["+separators+"]");
	String[] s2s = s2.split("["+separators+"]");
	for(int i=0; i<s1s.length && i<s2s.length; i++){
	    if(s1s[i].equals(s2s[i]))
		ret += s1s[i]+File.separator; 
	    else
		break;
	}
	return ret;
    }

    /**
     * Used by infomat.algorithms.LocationGrouper
     */
    public static String removeLastPart(String s){
	String[] ll = s.split("["+separators+"]");
	String ret = "";
	for(int i=0; i<ll.length-1; i++){
	    ret += ll[i]+File.separator; 
	}
	return ret;
    }





}
