package mro.util;

import java.io.*;

/**
 * Interface for classes that are easily handled as parts of XML files. 
 *
 * @version          061130
 * @author           Magnus Rosell
 */
public interface XMLPartIO{

    /**
     * Prints the DTD for a UniqueIndentifier. 
     * Use in the context of a larger DTD.
     */
    public void printPartDTD(PrintWriter out);

    /**
     * Prints to a PrintWriter as a part of a larger
     * XML-file following the DTD in {@link #printPartDTD}.
     */
    public void printPartXML(PrintWriter out);


    /**
     * Use when defining a {@link org.xml.sax.helpers.DefaultHandler}
     * to parse a larger XML-file where the implementing class is a part.
     */
    public boolean startElementPartXML(StringBuffer buffer, 
				       String qualifiedName);

    /**
     * Use when defining a {@link org.xml.sax.helpers.DefaultHandler}
     * to parse a larger XML-file where the implementing class is a part.
     * Returns true when the elements in the DTD 
     * (see {@link #printPartDTD}) are found.
     * In these cases the content of the implemented class should be 
     * set to the the values found in the XML-file.
     */
    public boolean endElementPartXML(StringBuffer buffer, 
				     String qualifiedName);
}
