package mro.util.experimentation;

import mro.util.*;
import mro.util.doublehashmap.*;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.SAXException;


/**
 * Path Table. A class for handling a xml-table of paths.
 *
 * @version          080505
 * @author           Magnus Rosell
 */

public class PathTable{
    
    private String commonPath = "";

    private GenericOrderedDoubleHashMap<Integer, Integer, PathEntry> map 
	= new GenericOrderedDoubleHashMap<Integer, Integer, PathEntry>();

    public void put(int r, int c, PathEntry pe){
	map.put(r, c, pe);
    }

    /*
    public void put(int r, int c, String path){
	Vector<String> vec = map.get(r,c);
	if(vec==null){
	    vec = new Vector<String>();
	    vec.add(path);
	    map.put(r, c, vec);
	}else{
	    vec.add(path);
	}
    }
    */

    public String getCommonPath(){
	return commonPath;
    }

    public void setCommonPath(String commonPath){
	this.commonPath = commonPath;
    }

    public GenericOrderedDoubleHashMap<Integer, Integer, PathEntry> 
	getGenericOrderedDoubleHashMap(){

	return new GenericOrderedDoubleHashMap<Integer, Integer, PathEntry>(map);
    }

    public Vector<String> getAllPathsWithCommonPath(){
	Vector<String> ret = new Vector<String>();
	Vector<Integer> first = map.getFirstIndexOrderVector();
	Vector<Integer> second = map.getSecondIndexOrderVector();
	for(int f=0; f<first.size(); f++){
	    for(int s=0; s<second.size(); s++){
		//		Vector<String> paths = 
		//		    map.get(first.elementAt(f), second.elementAt(s));
		PathEntry pe = 
		    map.get(first.elementAt(f), second.elementAt(s));
		Vector<String> paths = pe.getPaths();

		for(int i=0; i<paths.size(); i++){
		    String p = "";
		    if(!commonPath.equals("")) 
			p += commonPath + File.separator;
		    p += paths.elementAt(i);
		    ret.add(p);
		}
	    }
	}
	return ret;
    }

    public void print(){
	map.print();
    }


    /********* IO *********/

    /**
     * Saves a Table-object to a file with the specified file name.
     */
    public void save(String fileName) throws IOException{
	save(new File(fileName));
    }

    /**
     * Saves a Table-object to the specified file.
     */
    public void save(File file) throws IOException{
	//	try {
	    File dir = file.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

	    // XSL
	    String xslFileName = file.getParent()+File.separator+
		"PathTable.xsl";

	    PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFileName))));
	    printXSL(out);
	    out.close();

	    // XML
	    out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(file))));

	    //            printXMLHead(out);
	    printXML(out);
	    out.close();
	    /*
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	    */
    }

    /**
     * Prints the complete DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printDTD(PrintWriter pw) throws IOException{
	pw.println("<!DOCTYPE pathtable [");
	printPartDTD(pw);

	pw.println("]>");
    }

    /**
     * Prints the DTD of a Table-XML-file to a 
     * {@link java.io.PrintWriter}. Only the elements.
     * This method may be used when the Table-XML should
     * be part of a larger XML.
     */
    public static void printPartDTD(PrintWriter pw) throws IOException{
	pw.println("<!--A path table-->");
	pw.println("<!ELEMENT pathtable (commonpath?, ptr*)>");

	pw.println("<!--A path common to all-->");
	pw.println("<!ELEMENT commonpath (#PCDATA)>");

	pw.println("<!--A path table row-->");
	pw.println("<!ELEMENT ptr (ptc*)>");

	pw.println("<!--A path table column-->");
	pw.println("<!ELEMENT ptc (p+)>");

	pw.println("<!--A path-->");
	pw.println("<!ELEMENT p (#PCDATA)>");
    }

    /**
     * Prints a reference to the Table-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printTableXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\"PathTable.xsl\"?>");
    }

    /**
     * Prints the Table to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXML(PrintWriter pw) throws IOException{
	XMLHelper.printXMLHead(pw);
	printTableXSLReference(pw); 
	printDTD(pw);
	printPartXML(pw);
    }

    public void printPartXML(PrintWriter pw) throws IOException{
	pw.println("<pathtable>");
	pw.print("<commonpath>");
	pw.print(commonPath);
	pw.println("</commonpath>");
	Vector<Integer> vec1 = map.getFirstIndexOrderVector();
	Vector<Integer> vec2 = map.getSecondIndexOrderVector();
	for(int i=0; i<vec1.size(); i++){
	    pw.print("<ptr>");
	    for(int j=0; j<vec2.size(); j++){
		pw.print("<ptc>");
		//		Vector<String> paths = 
		//		    map.get(vec1.elementAt(i), vec2.elementAt(j));

		PathEntry pe = 
		    map.get(vec1.elementAt(i), vec2.elementAt(j));
		pw.print("<m>");
		pw.print(pe.getMeasureAbbreviation());
		pw.print("</m>");

		Vector<String> paths = pe.getPaths();
		for(int k=0; k<paths.size(); k++){
		    pw.print("<p>");
		    pw.print(paths.elementAt(k));
		    pw.print("</p>");
		}
		pw.print("</ptc>");
	    }
	    pw.println("</ptr>");
	}
	pw.println("</pathtable>");
    }


    /**
     * Prints a basic XSL-file for Table XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<table border=\"1\">");

	pw.println("<xsl:apply-templates/>");

	pw.println("</table>");
	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");


	pw.println("<xsl:template match=\"ptr\">");
	pw.println("<tr>");
	pw.println("<xsl:for-each select=\"ptc\">");
	pw.print("<td><xsl:value-of select=\".\"/></td>");
	pw.println("\n</xsl:for-each>");
	pw.println("</tr>");
	pw.println("</xsl:template>");

	pw.println("</xsl:stylesheet>");

    }

}

