package mro.util.measures;

import mro.util.*;

import infomat.vectorspace.IObjectGroup;

import java.io.*;
import java.util.*; 

/**
 * A group of AverageMeasure:s for the purpose of calcuation.
 *
 * @version          080507
 * @author           Magnus Rosell
 */
public class AverageMeasureGroup extends AverageMeasure{

    /**
     * The vector of AverageMeasure:s.
     */
    Vector<AverageMeasure> averageMeasureVector = 
	new Vector<AverageMeasure>();

    public AverageMeasureGroup(String name, String description){
	this.name=name;
	this.description=description;
	this.measureVector=null;
    }


    /**
     * Copy Constructor. Deep (the measure vectors is duplicated 
     * with new Measure:s).
     */
    public AverageMeasureGroup(AverageMeasureGroup am2){
	this.name=am2.name;
	this.value=am2.value;
	this.std=am2.std;
	this.description=am2.description;
	this.stamp=am2.getStamp();
	this.measureVector=null;
	this.averageMeasureVector=new Vector<AverageMeasure>();
	for(int i=0; i<am2.averageMeasureVector.size(); i++)
	    this.averageMeasureVector.add(new AverageMeasure(am2.averageMeasureVector.elementAt(i)));
    }

    protected Vector<AverageMeasure> getAverageMeasureVector(){
	return averageMeasureVector;
    }

    /**
     * Copies the AverageMeasure:s.
     */
    public void addAverageMeasureVector(Vector<AverageMeasure> vec){
	for(int i=0; i<vec.size(); i++){
	    addAverageMeasure(new AverageMeasure(vec.elementAt(i)));
	}
    }

    public boolean addAverageMeasure(AverageMeasure am){
	if(am instanceof AverageMeasure){
	    averageMeasureVector.add(am);
	    return true;
	}
	return false;
    }

    protected void sortAllMeasureVectorsInStampOrder(){
	for(int i=0; i<averageMeasureVector.size(); i++)
	    averageMeasureVector.elementAt(i).sortMeasureVectorInStampOrder();
    }

    public AverageMeasureGroup subtract(AverageMeasureGroup other){
	if(this.averageMeasureVector.size() != 
	   other.averageMeasureVector.size())
	    return null;
	
	other.sortAllMeasureVectorsInStampOrder();
	AverageMeasureGroup amg = new AverageMeasureGroup(this);
	amg.sortAllMeasureVectorsInStampOrder();
	for(int i=0; i<amg.averageMeasureVector.size(); i++){
	    amg.averageMeasureVector.set(i, amg.averageMeasureVector.elementAt(i).subtract(other.averageMeasureVector.elementAt(i)));
	}
	return amg;
    }

    public AverageMeasureGroup divide(AverageMeasureGroup other){
	if(this.averageMeasureVector.size() != 
	   other.averageMeasureVector.size())
	    return null;
	
	other.sortAllMeasureVectorsInStampOrder();
	AverageMeasureGroup amg = new AverageMeasureGroup(this);
	amg.sortAllMeasureVectorsInStampOrder();
	for(int i=0; i<amg.averageMeasureVector.size(); i++){
	    amg.averageMeasureVector.set(i, amg.averageMeasureVector.elementAt(i).divide(other.averageMeasureVector.elementAt(i)));
	}
	return amg;
    }



    /**
     * Really slow.
     */
    private void calculate(){
	value=0.0;
	std=0.0;
	for(int i=0; i<averageMeasureVector.size(); i++){
	    value += averageMeasureVector.elementAt(i).getValue();
	}
	value /= averageMeasureVector.size();


	std=0.0;
	for(int i=0; i<averageMeasureVector.size(); i++){
	    std += Math.pow(averageMeasureVector.elementAt(i).getSTD(), 2);
	}
	std = Math.sqrt(std);
    }

    public double getValue(){
	calculate();
	return value;
    }
    
    public double getSTD(){
	calculate();
	return std;
    }
    
    public void print(){
	calculate();
	System.out.println(name+": "+value+", "+std);
    }


    public void printXML(PrintWriter pw) throws IOException{
	calculate();
	pw.println("<amg>");
	pw.print("<n>"+getName()+"</n>");
	pw.println("<v>"+getValue()+"</v>");
	pw.println("<std>"+getSTD()+"</std>");
	pw.println("<d>"+getDescription()+"</d>");
	pw.println("<stamp>"+getStamp()+"</stamp>");
	for(int i = 0; i < averageMeasureVector.size(); i++){
	    averageMeasureVector.get(i).printXML(pw);
	}
	pw.println("</amg>");
    }

}


