package mro.util.measures;

import mro.util.*;

import infomat.vectorspace.IObjectGroup;

import java.io.*;
import java.util.*; 

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;



/********
 * @see Measure
 * @see MeasureGroup
 *
 * @version          080326
 * @author           Magnus Rosell
 */

public class Measures extends MeasureGroup{
    
    /**
     * A constructor.
     */
    public Measures(String name){
	this.name=name;
    }

    /**
     * Shallow copy constructor.
     */
    public Measures(MeasureGroup mg) throws MeasureNameConflictException{
	super(mg);
    }


    /**
     * Copy constructor.
     */
    /*
      public Measures(Measures ps) throws MeasureNameConflictException{
      measuresName=ps.measuresName;
      ownerIdentifier=ps.ownerIdentifier;

      if(ps.relatedIdentifiers!=null){
      relatedIdentifiers = new Vector<UniqueIdentifier>();
      for(int i=0; i<ps.relatedIdentifiers.size(); i++){
      relatedIdentifiers.add(ps.relatedIdentifiers.elementAt(i));
      }
      }

      // The measures
      for(int i=0; i<ps.measureVector.size(); i++){
      Measure p = (Measure) ps.measureVector.elementAt(i);

      if(p instanceof MeasureGroup){
      MeasureGroup pg =(MeasureGroup) p;
      MeasureGroup newPG=new MeasureGroup(pg);
      addMeasure(newPG);
      }else{
      Measure newP = new Measure(p);
      addMeasure(newP);
      }
      }
      }
    */

    /**
     * Adds a {@link Measure} (or a {@link MeasureGroup}) to the measures.
     */
    /*
    public void addMeasure(Measure p) throws MeasureNameConflictException{
	if(p instanceof MeasureGroup){
	    MeasureGroup pg =(MeasureGroup) p;
	    if(getMeasureGroup(pg.getName())!=null){
		System.out.println(pg.getName());
		throw new MeasureNameConflictException();
	    }else{
		nameMeasureGroupMap.put(pg.getName(), pg);
		Vector measures = pg.getMeasureVector();
		recursivelyAddmeasuresInMeasureVector(measures);
	    }
	}else{
	    if(getMeasure(p.getName())!=null){
		throw new MeasureNameConflictException();
	    }else{
		nameMeasureMap.put(p.getName(), p);
	    }
	}
	measureVector.add(p);
    }
    */

    /**
     * Used by the {@link #addMeasure}-method.
     * The meausres within groups can not be reached through
     * the "nameMeasureMap".
     */
    /*
    private void recursivelyAddmeasuresInMeasureVector(Vector measures)
	throws MeasureNameConflictException{

	for(int i=0; i<measures.size(); i++){
	    Measure p = (Measure) measures.elementAt(i);
	    if(p instanceof MeasureGroup){
		recursivelyAddmeasuresInMeasureVector(((MeasureGroup) p).getMeasureVector());
	    }else{
//		if(getMeasure(p.getName())!=null){
//		    throw new MeasureNameConflictException();
//		}else{
		nameMeasureMap.put(p.getName(), p);
		    //		}
	    }
	}
    }
    */

    /*
    public String getMeasuresName(){
	return name;
    }
    */
	
    /********* IO *********/

    /**
     * Saves a measures-object to a file with the specified file name.
     */
    public void save(String fileName) {
	save(new File(fileName));
    }

    public void save(File f){
	try {
	    File dir = f.getParentFile();
	    if(!dir.exists())
		dir.mkdirs();

	    // XSL
	    String xslFileName = f.getParent()+"/measures.xsl";
	    PrintWriter out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(xslFileName))));
	    printXSL(out);
	    out.close();

	    // XML
	    String fileName = f.getCanonicalPath();
	    if(!fileName.endsWith(".xml"))
		fileName += ".xml";
	    out = new PrintWriter(new BufferedWriter(new BufferedWriter(new FileWriter(fileName))));

	    //            printXMLHead(out);
	    printXML(out);
	    out.close();
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
    }

    /**
     * Prints the complete DTD of a measures-XML-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printDTD(PrintWriter pw) throws IOException{
	pw.println("<!DOCTYPE measures [");
	printPartDTD(pw);

	pw.println("]>");
    }

    /**
     * Prints the DTD of a measures-XML-file to a 
     * {@link java.io.PrintWriter}. Only the elements.
     * This method may be used when the measures-XML should
     * be part of a larger XML.
     */
    public static void printPartDTD(PrintWriter pw) throws IOException{
	pw.println("<!ELEMENT measures (n, d, stamp, MList)>");


	pw.println("<!--The List of Measure:s-->");
	pw.println("<!ELEMENT MList (mg|(m|mm|am))*>");

	pw.println("<!--A Group of Measure:s-->");
	pw.println("<!ELEMENT mg (n, d, stamp, (m|mm|am)*)>");

	pw.println("<!--A MultiMeasure-->");
	pw.println("<!ELEMENT mm (n, v, d, stamp, m*)>");
		
	pw.println("<!--A AverageMeasure-->");
	pw.println("<!ELEMENT am (n, v, std, d, stamp, m*)>");
	pw.println("<!ELEMENT std (#PCDATA)> <!--The standard deviation of the Measure-->");
		
	pw.println("<!--A Measure-->");
	pw.println("<!ELEMENT m (n, v, d, stamp)>");

	pw.println("<!ELEMENT n (#PCDATA)> <!--The name of the Measure-->");
	pw.println("<!ELEMENT v (#PCDATA)> <!--The value of the Measure-->");
	pw.println("<!ELEMENT d (#PCDATA)> <!--A description of the Measure-->");
	pw.println("<!ELEMENT stamp (#PCDATA)> <!--A identifier for the Measure-->");

    }

    /**
     * Prints a reference to the measures-XSL-file to a 
     * {@link java.io.PrintWriter}.
     */
    public static void printmeasuresXSLReference(PrintWriter pw) throws IOException{
	pw.println("<?xml-stylesheet type=\"text/xsl\" href=\"measures.xsl\"?>");
    }

    /**
     * Prints the measures to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXML(PrintWriter pw) throws IOException{
	XMLHelper.printXMLHead(pw);
	printmeasuresXSLReference(pw); 
	printDTD(pw);
	printPartXML(pw);
    }
	
    public void printOut(){
	Vector<Measure> measureVector = getMeasureVector();
	for(int i = 0; i < measureVector.size(); i++){
	    System.out.println("Name: " + measureVector.get(i).getName() + " Value: " + measureVector.get(i).getValue());
	}
    }

    public void printPartXML(PrintWriter pw) throws IOException{
	pw.println("<measures>");
	pw.println("<n>"+getName()+"</n>");
	pw.println("<d>"+getDescription()+"</d>");
	pw.println("<stamp>"+getStamp()+"</stamp>");

	pw.println("<MList>");
	Vector<Measure> measureVector = getMeasureVector();
	for(int i=0; i<measureVector.size(); i++){
	    measureVector.elementAt(i).printXML(pw);
	}
	pw.println("</MList>");
	pw.println("</measures>");
    }


    /**
     * Prints a basic XSL-file for measures XML-files to a 
     * {@link java.io.PrintWriter}.
     */
    public void printXSL(PrintWriter pw) throws IOException{
	pw.println("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
	pw.println("<xsl:stylesheet version=\"1.0\"");
	pw.println("xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\">");
	pw.println("<xsl:template match=\"/\">");
	pw.println("<html>");
	pw.println("<body>");
	pw.println("<xsl:apply-templates/>");
	pw.println("</body>");
	pw.println("</html>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"n\">");
	pw.println("<h2><xsl:value-of select=\".\"/></h2>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"d\">");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"stamp\">");
	pw.println("</xsl:template>");
	pw.println("");
	//		pw.println("<xsl:template match=\"Identifiers\">");
	//		pw.println("<h3>Identifiers</h3>");
	//		pw.println("<b>Owner: </b><xsl:value-of select=\"./Owner/UniqueIdentifier/OwnerClassName\"/><br/>");
	//		pw.println("<xsl:apply-templates select=\"Related\"/>");
	//		pw.println("</xsl:template>");
	//		pw.println("");
	//		pw.println("<xsl:template match=\"Related\">");
	//		pw.println("<b>Related: </b>");
	//		pw.println("<xsl:for-each select=\"./UniqueIdentifier\">");
	//		pw.println("<xsl:value-of select=\"OwnerClassName\"/>");
	//		pw.println("<xsl:if test=\"position()!=last()\">");
	//		pw.println("<xsl:text>, </xsl:text>");
	//		pw.println("</xsl:if>");
	//		pw.println("<xsl:if test=\"position()=last()-1\">");
	//		pw.println("<xsl:text> and </xsl:text>");
	//		pw.println("</xsl:if>");
	//		pw.println("</xsl:for-each>");
	//		pw.println("<br/>");
	//		pw.println("</xsl:template>");
	pw.println("");
	pw.println("<xsl:template match=\"MList\">");
	pw.println("<h3>Measures</h3>");
	pw.println("<table border=\"1\">");
	pw.println("<xsl:apply-templates select=\"m\"/>");
	pw.println("<xsl:apply-templates select=\"mg\"/>");
	pw.println("<xsl:apply-templates select=\"am\"/>");
	pw.println("</table>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"mg\">");
	pw.println("<p>");
	pw.println("<b><xsl:value-of select=\"n\"/></b><br/>");
	pw.println("<table border=\"1\">");
	pw.println("<xsl:apply-templates select=\"m\"/>");
	pw.println("<xsl:apply-templates select=\"mg\"/>");
	pw.println("<xsl:apply-templates select=\"am\"/>");
	pw.println("</table>");
	pw.println("</p>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("<xsl:template match=\"am\">");
	pw.println("<tr>");
	pw.println("<td>Average <xsl:value-of select=\"n\"/></td><td><xsl:value-of select=\"v\"/></td><td><xsl:value-of select=\"std\"/></td><td><xsl:value-of select=\"nr\"/></td>");
	pw.println("</tr>");
	pw.println("</xsl:template>");
	pw.println("");
	pw.println("");

	pw.println("<xsl:template match=\"m\">");
	pw.println("<tr>");
	pw.println("<td><xsl:value-of select=\"n\"/></td><td><xsl:value-of select=\"v\"/></td>");
	pw.println("</tr>");
	pw.println("</xsl:template>");
	pw.println("");

	pw.println("</xsl:stylesheet>");
    }


    /**
     * Loads a measures-object from a file with the specified file name.
     */
    public static Measures load(String fileName) {
	File file = new File(fileName);
	return load(file);
    }

    /**
     * Loads a measures-object from the specified file.
     */
    public static Measures load(File file) {
	Measures measures = null;
	try {
	    BufferedReader in = new BufferedReader(new FileReader(file));
	    InputSource source = new InputSource(in);

	    //	    measures iop = new measures("Dummy");
	    measures=loadXML(source);
	}
	catch(IOException ioE) {
	    System.out.println(ioE);
	    ioE.printStackTrace();
	}
	return measures;
    }



    /**
     * Loads a measures-object from an {@link java.io.InputSource}.
     */
    public static Measures loadXML(InputSource source){
	Measures measures = null;
	try {
	    // Create parser+contentHandler and read the XML document
	    XMLReader parser = XMLReaderFactory.createXMLReader();
	    MeasuresHandler handler = new MeasuresHandler();
	    parser.setContentHandler(handler);
	    parser.parse(source);
	    measures = handler.getmeasures();
	} catch (Exception e) {
	    System.err.println(e);
	    e.printStackTrace();
	}
	return measures;
    }


}

/**
 * Class used for XML-parsing of {@link Measures}-objects.
 */
class MeasuresHandler extends DefaultHandler {

    StringBuffer buffer;

    Measures measures=null;
    Stack<MeasureGroup> measureGroupStack = new Stack<MeasureGroup>();

    // For one Measure
    String name=null;
    String value=null;
    String description=null;
    String stamp=null;

    int currentType = -1; // 0 - Measures, 1 - MeasureGroup, 2 - Measure

    public Measures getmeasures(){
	return measures;
    }

    public void startElement (String namespaceURI, String localName,
			      String qualifiedName, Attributes atts) {
	buffer = new StringBuffer();

	if(qualifiedName.equals("measures")){
	    currentType=0;
	}
	else if(qualifiedName.equals("mg")){
	    currentType=1;
	    measureGroupStack.push(new MeasureGroup());
	}
	else if(qualifiedName.equals("m")){
	    currentType=2;
	}
    }


    public void endElement (String namespaceURI, String localName, String qualifiedName) {

	if(qualifiedName.equals("n")){
	    name=buffer.toString();
	    if(currentType==0)
		measures = new Measures(name);
	    if(currentType==1)
		measureGroupStack.peek().setName(name);
	}
	else if(qualifiedName.equals("v")){
	    value=buffer.toString();
	}
	else if(qualifiedName.equals("d")){
	    description=buffer.toString();
	    if(currentType==0)
		measures.setDescription(description);
	    if(currentType==1)
		measureGroupStack.peek().setDescription(description);
	}
	else if(qualifiedName.equals("stamp")){
	    stamp=buffer.toString();
	    if(currentType==0)
		measures.setStamp(stamp);
	    if(currentType==1)
		measureGroupStack.peek().setStamp(stamp);
	}

	// En ny Measure
	else if(qualifiedName.equals("m")){
	    Measure m = new Measure(name, value, description, stamp);
	    try{
		if(measureGroupStack.size()>0){
		    measureGroupStack.peek().addMeasure(m);
		}else{
		    measures.addMeasure(m);
		}
	    }catch(MeasureNameConflictException me){
		System.out.println("Conflicting MeasureName: "+m.getName());
		System.out.println("Aborting!");
		System.exit(0);
	    }
	}
	
	// En MeasureGroup
	else if(qualifiedName.equals("mg")){
	    MeasureGroup mg = measureGroupStack.pop();
	    try{
		if(measureGroupStack.size()>0)
		    measureGroupStack.peek().addMeasure(mg);
		else
		    measures.addMeasure(mg);
	    }catch(MeasureNameConflictException me){
		System.out.println("Conflicting MeasureName: "+mg.getName());
		System.out.println("Aborting!");
		System.exit(0);
	    }
	}
    }

    public void characters (char ch[], int start, int length) {
	if(buffer != null)
	    buffer.append(ch,start,length);
    }      
}

