package mro.util.measures.dialog;

import mro.util.measures.*;

import java.io.*;

import java.util.Vector;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

import org.xml.sax.SAXException;


/**
 * A JPanel that displays a Measures.
 *
 * @version          080619
 * @author           Magnus Rosell
 *
 */
public class MeasuresPanel extends JPanel
    implements ActionListener{

    private Measures measures;

    private boolean fileable=false;
    private FilePanel filePanel;
    private InfoPanel infoPanel;

    private HashMap<JButton, Measure> buttonMeasureMap;

    private Color measureBorderColor = Color.BLACK;

    private JFileChooser fileChooser;
    private String fileChooserPath="";

    private MeasureMeasureGroupSorter measureMeasureGroupSorter
	= new MeasureMeasureGroupSorter();

    public MeasuresPanel(String path){
	initialize(null, path);
    }

    public MeasuresPanel(){
	initialize(null, null);

	/*
	this.measures=null;
	buttonMeasureMap = new HashMap<JButton, Measure>();

	fileChooserPath = ""; //(String) model.getMeasureValue("Matrix File Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });


	//setLayout(new GridLayout(measures.size(), 1));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	setPreferredSize(MeasuresFrame.MEASURESFRAME_PREFERRED_SIZE);

	infoPanel = new InfoPanel();
	
	putAll();
	*/
    }

    public MeasuresPanel(Measures measures, String path){
	initialize(measures, path);
    }

    public MeasuresPanel(Measures measures){
	initialize(measures, null);
	/*
	this.measures=measures;
	buttonMeasureMap = new HashMap<JButton, Measure>();

	fileChooserPath = ""; //(String) model.getMeasureValue("Matrix File Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });


	//setLayout(new GridLayout(measures.size(), 1));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	setPreferredSize(MeasuresFrame.MEASURESFRAME_PREFERRED_SIZE);
	setBorder(new TitledBorder(measures.getName()));

	infoPanel = new InfoPanel();
	
	putAll();
	*/
    }



    private void initialize(Measures measures, String path){
	this.measures=measures;
	buttonMeasureMap = new HashMap<JButton, Measure>();

	fileChooserPath = "";
	if(path!=null)
	    fileChooserPath = path;
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });


	//setLayout(new GridLayout(measures.size(), 1));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	setPreferredSize(MeasuresFrame.MEASURESFRAME_PREFERRED_SIZE);
	
	if(measures!=null)
	    setBorder(new TitledBorder(measures.getName()));

	infoPanel = new InfoPanel();
	
	putAll();
    }

    public void associate(JButton button, Measure p){
	buttonMeasureMap.put(button, p);
    }

    public Color getMeasureBorderColor(){
	return measureBorderColor;
    }

    public void setFileable(boolean fileable){
	this.fileable=fileable;
	putAll();
    }

    public void putAll(){
	if(measures==null)
	    return;

	removeAll();

	if(fileable){
	    filePanel = new FilePanel(this);
	    
	    filePanel.setMinimumSize(filePanel.getPreferredSize());
	    filePanel.setMaximumSize(filePanel.getPreferredSize());
	    add(filePanel);
	}

	JPanel msPanel = new JPanel();
	msPanel.setLayout(new BoxLayout(msPanel, BoxLayout.Y_AXIS));
	Vector<Measure> ms = 
	    measures.getMeasureVector();
	ms = measureMeasureGroupSorter.sort(ms);
	for(int i=0; i<ms.size(); i++){
	    Measure p = ms.elementAt(i);
	    if(p instanceof MeasureGroup)
		msPanel.add(new MeasureGroupPanel((MeasureGroup) p, this));
	    else
		msPanel.add(new MeasurePanel(p, this));
	}
	JScrollPane sp = new JScrollPane();
	sp.setViewportView(msPanel);
	sp.setBorder(new LineBorder(getMeasureBorderColor()));
	//	sp.setBorder(new TitledBorder(""));
	add(sp);

	add(infoPanel);


	revalidate();
	repaint();
    }

    private void showMessage(String title, String message){
	JOptionPane.showMessageDialog(
				      this, message, title, 
				      JOptionPane.INFORMATION_MESSAGE);

    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();

	    if(fileable && source == filePanel.getLoadButton()){
		
		fileChooser.setDialogTitle("Load Measures");
		fileChooser.setSelectedFile(new File(""));
		if (fileChooser.showDialog(getParent(), "Load") == JFileChooser.APPROVE_OPTION) {
		    File file = fileChooser.getSelectedFile();
		    Measures measures = Measures.load(file);
		    this.measures = measures;
		    putAll();
		}


	    }else if(fileable && source == filePanel.getSaveButton()){

		fileChooser.setDialogTitle("Save Measures");
		fileChooser.setSelectedFile(new File(""));
		if (fileChooser.showDialog(getParent(), "Save") == JFileChooser.APPROVE_OPTION) {
		    try{
			File file = fileChooser.getSelectedFile();
			if(!file.getCanonicalPath().endsWith(".xml"))
			    file = new File(file.getCanonicalPath()+".xml");
			measures.save(file);
		    }catch(IOException ioe){
			showMessage("IOException", "An IOException occured when trying to save.");
		    }
		}

		/*
	    }else if(source == filePanel.getDefaultButton()){
		measures.setMeasures(measures.getDefaultMeasures());
		putAll();
		*/

	    }else{

		Iterator<Map.Entry<JButton, Measure>> it = 
		    buttonMeasureMap.entrySet().iterator();
		while(it.hasNext()){
		    Map.Entry<JButton, Measure> entry = it.next();
		    JButton button = entry.getKey();
		    if(source == button){
			Measure p = entry.getValue();
			infoPanel.setMeasure(p);
			break;
		    }
		}
	    }


	}
    }

    public void paintcomponent(Graphics g){
	super.paintComponent(g);
    }

    
    private class MeasureMeasureGroupSorter{
	
	public Vector<Measure> sort(Vector<Measure> ms){
	    Vector<Measure> ps = new Vector<Measure>(ms);
	    Vector<Measure> pgs = new Vector<Measure>();
	    for(int i=0; i<ms.size(); i++){
		Measure p = ms.elementAt(i);
		if(p instanceof MeasureGroup){
		    pgs.add(p);
		    ps.remove(p);
		}
	    }
	    Vector<Measure> ret = new Vector<Measure>();
	    ret.addAll(ps);
	    ret.addAll(pgs);
	    return ret;
	}

    }



    private class FilePanel extends JPanel{
	private MeasuresPanel measuresPanel;
	private JButton load;
	private JButton save;
	//	private JButton def;

	public FilePanel(MeasuresPanel measuresPanel){
	    this.measuresPanel=measuresPanel;

	    setLayout(new FlowLayout(FlowLayout.CENTER));

	    load = new SmallButton("Load");
	    load.addActionListener(measuresPanel);
	    add(load);

	    save = new SmallButton("Save");
	    save.addActionListener(measuresPanel);
	    add(save);

	    /*
	    def = new SmallButton("Default");
	    def.addActionListener(measuresPanel);
	    add(def);
	    */

	    //	    add(new Label(" ")); //new Label("Spara, ladda, default"));
	    setBorder(new TitledBorder("File"));
	    //	    setBorder(new LineBorder(getMeasureBorderColor()));
	    /*
	    int width = (int) getPreferredSize().getWidth();
	    int height = (int) load.getPreferredSize().getHeight();
	    Dimension d = new Dimension(width, 3 * height);
	    setPreferredSize(d);
	    setMinimumSize(d);
	    setMaximumSize(d);
	    */

	    revalidate();
	    repaint();
	}

	public JButton getLoadButton(){
	    return load;
	}

	public JButton getSaveButton(){
	    return save;
	}

	/*
	public JButton getDefaultButton(){
	    return def;
	}
	*/
    }

    private class InfoPanel extends JPanel{
	public Measure measure;
	public InfoPanel(){
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setBorder(new TitledBorder("Description"));
	    putAll();
	}

	private void putAll(){
	    removeAll();

	    JPanel infoPanel = new JPanel();
	    
	    if(measure==null)
		infoPanel.add(new JLabel("Press measure buttons for info.")); //new Label("Info"));
	    else{
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));

		/*		
		JLabel title = new JLabel(measure.getName());
		infoPanel.add(title);
		*/
		LeftLabel title = new LeftLabel(measure.getName());
		infoPanel.add(title);

		/*		
		JLabel description = new JLabel(measure.getDescription());
		infoPanel.add(description);
		*/

		LeftLabel description = new LeftLabel(measure.getDescription());
		infoPanel.add(description);
		

		
	    }

	    JScrollPane sp = new JScrollPane();
	    sp.setViewportView(infoPanel);
	    sp.setBorder(new LineBorder(getMeasureBorderColor()));
	    add(sp);

	    int width = (int) getPreferredSize().getWidth();
	    int height = (int) (new JLabel("Dummy")).getPreferredSize().getHeight();
	    if(measure==null)
		height = 4 * height;
	    else{
		height = 5 * height;
	    }


	    //	    d = new Dimension(MeasuresFrame.MEASURESFRAME_PREFERRED_SIZE.width, (int) (3*d.getHeight()));
	    Dimension d = new Dimension(width, height);
	    setPreferredSize(d);
	    setMinimumSize(d);


	    revalidate();
	    repaint();
	}

	public void setMeasure(Measure measure){
	    this.measure=measure;
	    putAll();
	}

	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}
    }

    private class LeftLabel extends JPanel{
	public LeftLabel(String s){
	    setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel l = new JLabel(s);
	    add(l);
	    //	    setPreferredSize(l.getPreferredSize());
	    JTextField dummy = new JTextField(s);
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) l.getPreferredSize().getWidth();
	    Dimension size = new Dimension(width, height);
	    setPreferredSize(size);
	}
    }


    private class MeasureGroupPanel extends JPanel{
	private MeasuresPanel measuresPanel;
	private MeasureGroup pg;

	private MeasureMeasureGroupSorter measureMeasureGroupSorter
	    = new MeasureMeasureGroupSorter();


	public MeasureGroupPanel(MeasureGroup pg, 
				  MeasuresPanel measuresPanel){
	    this.pg=pg;
	    this.measuresPanel=measuresPanel;

	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    putAll();
	}
	    
	private void putAll(){
	    removeAll();
	    String name = pg.getName();

	    JPanel namePanel = new JPanel();
	    namePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
 	    JLabel label = new JLabel("*** "+name);
	    namePanel.add(label);
	    add(namePanel);

	    Font f = label.getFont();
	    label.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));
	    
	    /*
 	    JLabel label = new JLabel("*** "+name, JLabel.LEFT);
	    add(label);
	    */

	    Vector<Measure> ms = pg.getMeasureVector();
	    ms = measureMeasureGroupSorter.sort(ms);
	    for(int i=0; i<ms.size(); i++){
		Measure p = ms.elementAt(i);
		if(p instanceof MeasureGroup)
		    add(new MeasureGroupPanel((MeasureGroup) p, measuresPanel));
		else
		    add(new MeasurePanel(p, measuresPanel));
	    }

	    revalidate();
	    repaint();
	}


	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}
    }

    private class MeasurePanel extends JPanel{

	private Measure measure;
	private JButton nameButton;
	private JLabel valueLabel;

	private MeasuresPanel measuresPanel;
		
	public MeasurePanel(Measure measure, 
			    MeasuresPanel measuresPanel){
	    this.measure=measure;
	    this.measuresPanel=measuresPanel;
	    int numberOfParts = 3;
			
	    //setLayout(new GridLayout(numberOfParts, 1));
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setBorder(new LineBorder(measuresPanel.getMeasureBorderColor()));

	    nameButton = new SmallButton(measure.getName());
	    nameButton.addActionListener(measuresPanel);
	    measuresPanel.associate(nameButton, measure);

	    putAll();
	}
	
	private void putAll(){
	    removeAll();
	    
	    JPanel namePanel = new JPanel();
	    namePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    namePanel.add(nameButton);
	    add(namePanel);


	    JPanel valuePanel = new JPanel();
	    valuePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    String value = ""+measure.getValue();
	    valueLabel = new JLabel("Value: "+value+", ");
	    valuePanel.add(valueLabel);

	    add(valuePanel);

	    revalidate();
	    repaint();
	}

	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}
		
    }

    private class SmallButton extends JButton{

	public SmallButton(String string){
	    super(string);
	    JTextField dummy = new JTextField(string);
	    JButton dummyButton = new JButton(string);
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) dummyButton.getPreferredSize().getWidth();
	    Dimension buttonSize = new Dimension(width, height);
	    setPreferredSize(buttonSize);
	}

    }
    

	
}
