package mro.util.properties;

import java.io.*;
import java.util.*; 

/**
 *
 * @version          061128
 * @author           Magnus Rosell
 */
public class PropertyGroup extends Property{
    String propertyGroupName;
    Vector<Property> properties = new Vector<Property>();

    /** Info about the Properties (and its owning class). Not used yet.*/
    private String description=null;

    // Used in Properties
    PropertyGroup(){}

    public PropertyGroup(String propertyGroupName){
	this.propertyGroupName=propertyGroupName;
    }

    /**
     * Copy Constructor.
     */
    public PropertyGroup(PropertyGroup pg){
	this.propertyGroupName=pg.propertyGroupName;
	for(int i=0; i<pg.properties.size(); i++){
	    Property p = (Property) pg.properties.elementAt(i);
	    if(p instanceof PropertyGroup){
		addProperty(new PropertyGroup((PropertyGroup) p));
		//		addPropertyVector(((PropertyGroup) p).properties);
	    }else{
		addProperty(new Property(p));
	    }
	}
    }

    public void addProperty(Property p){
	properties.add(p);
    }

    public void addPropertyVector(Vector pv){
	for(int i=0; i<pv.size(); i++){
	    Property p = (Property) pv.elementAt(i);
	    properties.add(p);
	}
    }

    public String getPropertyGroupName(){
	return propertyGroupName;
    }

    public void setPropertyGroupName(String name){
	propertyGroupName=name;
    }

    public Vector<Property> getPropertyVector(){
	return properties;
    }


    /**
     * Returns true if all Property:s (including PropertyGroup:s)
     * have the same name and are in the same order, false otherwise. 
     * The values does not have to be the same.
     */
    public boolean sameProperties(PropertyGroup propertyGroup2){

	Vector<Property> propertyVector = getPropertyVector();
	Vector<Property> propertyVector2 = propertyGroup2.getPropertyVector();
	if(propertyVector.size() != propertyVector2.size())
	    return false;
	for(int i=0; i<propertyVector.size(); i++){
	    Property p = propertyVector.elementAt(i);
	    Property p2 = propertyVector2.elementAt(i);
	    if(p instanceof PropertyGroup){
		if(!(p2 instanceof PropertyGroup))
		    return false;
		if(!((PropertyGroup) p).sameProperties((PropertyGroup) p2))
		    return false;
	    }else if(p2 instanceof PropertyGroup){
		return false;
	    }else{
		if(p.getName().compareTo(p2.getName()) != 0)
		    return false;
	    }
	}
	return true;
    }


    public void printXML(PrintWriter pw) throws IOException{
	pw.println("<pg>");

	pw.println("<pgn>"+propertyGroupName+"</pgn>");


	// Skriver alla underegenskaper
	for(int i=0; i<properties.size(); i++){
	    Property p = (Property) properties.elementAt(i);
	    p.printXML(pw);
	}
	pw.println("</pg>");
    }


    public String toString(){
	return propertyGroupName;
    }



    /****************** OLD *****************/

    public String toHTMLString(){
	String s="";
	s=s+"<br />\n<b>"+propertyGroupName+"</b><br />\n";

	for(int i=0; i<properties.size(); i++){
	    Property p = (Property) properties.elementAt(i);
	    s=s+p.toHTMLString();
	}

	return s;
    }

    public String toEvaluationFileString(){
	String s="";
	
	s=s+"-\n"+propertyGroupName+"\n";

	for(int i=0; i<properties.size(); i++){
	    Property p = (Property) properties.elementAt(i);
	    s=s+p.toEvaluationFileString();
	}

	return s;
    }

    public String toInfoFileString(){
	String s="";

	s=s+"-\n"+propertyGroupName+"\n";

	// Visar alla underegenskaper
	for(int i=0; i<properties.size(); i++){
	    Property p = (Property) properties.elementAt(i);
	    s=s+p.toInfoFileString();
	}

	return s;
    }


}


