package mro.util.properties.dialog;

import mro.util.properties.*;

import java.io.*;

import java.util.Vector;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;

import org.xml.sax.SAXException;


/**
 * A JPanel that displyas the Property:s of a HasProperty-object.
 */
public class PropertiesPanel extends JPanel
    implements ActionListener{

    private HasProperties hasProperties;

    private FilePanel filePanel;
    private InfoPanel infoPanel;

    private HashMap<JButton, Property> buttonPropertyMap;

    private Color propertyBorderColor = Color.BLACK;

    private JFileChooser fileChooser;
    private String fileChooserPath="";

    private PropertyPropertyGroupSorter propertyPropertyGroupSorter
	= new PropertyPropertyGroupSorter();

    public PropertiesPanel(HasProperties hasProperties, String path){
	initialize(hasProperties, path);
    }

    public PropertiesPanel(HasProperties hasProperties){
	initialize(hasProperties, null);

	/*
	this.hasProperties=hasProperties;
	buttonPropertyMap = new HashMap<JButton, Property>();

	fileChooserPath = ""; //(String) model.getPropertyValue("Matrix File Path");
	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });


	//setLayout(new GridLayout(properties.size(), 1));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	setPreferredSize(PropertiesFrame.PROPERTIESFRAME_PREFERRED_SIZE);
	setBorder(new TitledBorder(hasProperties.getProperties().getPropertiesName()));

	infoPanel = new InfoPanel();

	putAll();
	*/
    }


    private void initialize(HasProperties hasProperties, String path){
	this.hasProperties=hasProperties;
	buttonPropertyMap = new HashMap<JButton, Property>();

	fileChooserPath = ""; 
	if(path!=null)
	    fileChooserPath = path;

	fileChooser = new JFileChooser(fileChooserPath);
	fileChooser.setFileFilter(new FileFilter() {
		public String getDescription() {
		    return "XML files";
		}

		public boolean accept(File f) {
		    if (f.isDirectory())
			return true;
		    int findresult = f.getName().indexOf(".xml");
		    int len = f.getName().length();
		    return findresult != -1 && findresult == len - 4;
		}

	    });


	//setLayout(new GridLayout(properties.size(), 1));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	setPreferredSize(PropertiesFrame.PROPERTIESFRAME_PREFERRED_SIZE);
	setBorder(new TitledBorder(hasProperties.getProperties().getPropertiesName()));

	infoPanel = new InfoPanel();

	putAll();
    }

    public void associate(JButton button, Property p){
	buttonPropertyMap.put(button, p);
    }

    public Color getPropertyBorderColor(){
	return propertyBorderColor;
    }

    public void putAll(){
	removeAll();

	filePanel = new FilePanel(this);
	add(filePanel);

	JPanel propsPanel = new JPanel();
	propsPanel.setLayout(new BoxLayout(propsPanel, BoxLayout.Y_AXIS));
	Vector<Property> props = 
	    hasProperties.getProperties().getPropertyVector();
	props = propertyPropertyGroupSorter.sort(props);
	for(int i=0; i<props.size(); i++){
	    Property p = props.elementAt(i);
	    if(p instanceof PropertyGroup)
		propsPanel.add(new PropertyGroupPanel((PropertyGroup) p, this));
	    else
		propsPanel.add(new PropertyPanel(p, this));
	}
	JScrollPane sp = new JScrollPane();
	sp.setViewportView(propsPanel);
	sp.setBorder(new LineBorder(getPropertyBorderColor()));
	//	sp.setBorder(new TitledBorder(""));
	add(sp);

	add(infoPanel);


	revalidate();
	repaint();
    }

    private void showMessage(String title, String message){
	JOptionPane.showMessageDialog(
				      this, message, title, 
				      JOptionPane.INFORMATION_MESSAGE);

    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() instanceof JButton){
	    JButton source = (JButton) e.getSource();
	    if(source == filePanel.getLoadButton()){
		
		fileChooser.setDialogTitle("Load "+hasProperties.getProperties().getPropertiesName());
		fileChooser.setSelectedFile(new File(""));
		if (fileChooser.showDialog(getParent(), "Load") == JFileChooser.APPROVE_OPTION) {
		    File file = fileChooser.getSelectedFile();
		    try{
			Properties properties = Properties.load(file);
			if(!properties.sameProperties(hasProperties.getProperties()))
			    showMessage("Bad Properties", "The loaded Properties are not identical to these Properties.\n(Identical to names, not values.)\nNot loaded!");
			else{
			    hasProperties.setProperties(properties);
			    putAll();
			}
		    }catch(IOException ioe){
			showMessage("IOException", "An IOException occured when loading.\nNot loaded!");
		    }catch(SAXException saxe){
			showMessage("Parser error", "A parser problem occured when loading.\nNot loaded!");
		    }
		}


	    }else if(source == filePanel.getSaveButton()){

		fileChooser.setDialogTitle("Save "+hasProperties.getProperties().getPropertiesName());
		fileChooser.setSelectedFile(new File(""));
		if (fileChooser.showDialog(getParent(), "Save") == JFileChooser.APPROVE_OPTION) {
		    try{
			File file = fileChooser.getSelectedFile();
			if(!file.getCanonicalPath().endsWith(".xml"))
			    file = new File(file.getCanonicalPath()+".xml");
			hasProperties.getProperties().save(file);
		    }catch(IOException ioe){
			showMessage("IOException", "An IOException occured when trying to save.");
		    }
		}

	    }else if(source == filePanel.getDefaultButton()){
		hasProperties.setProperties(hasProperties.getDefaultProperties());
		putAll();
	    }else{
		//		boolean ok = false;
		Iterator<Map.Entry<JButton, Property>> it = 
		    buttonPropertyMap.entrySet().iterator();
		while(it.hasNext()){
		    Map.Entry<JButton, Property> entry = it.next();
		    JButton button = entry.getKey();
		    if(source == button){
			Property p = entry.getValue();
			infoPanel.setProperty(p);
			//			ok=true;
			break;
		    }
		}
		/*
		if(ok)
		    putAll();
		*/
	    }


	}
    }

    public void paintcomponent(Graphics g){
	super.paintComponent(g);
    }

    
    private class PropertyPropertyGroupSorter{
	
	public Vector<Property> sort(Vector<Property> props){
	    Vector<Property> ps = new Vector<Property>(props);
	    Vector<Property> pgs = new Vector<Property>();
	    for(int i=0; i<props.size(); i++){
		Property p = props.elementAt(i);
		if(p instanceof PropertyGroup){
		    pgs.add(p);
		    ps.remove(p);
		}
	    }
	    Vector<Property> ret = new Vector<Property>();
	    ret.addAll(ps);
	    ret.addAll(pgs);
	    return ret;
	}

    }




    private class FilePanel extends JPanel{
	private PropertiesPanel propertiesPanel;
	private JButton load;
	private JButton save;
	private JButton def;

	public FilePanel(PropertiesPanel propertiesPanel){
	    this.propertiesPanel=propertiesPanel;

	    setLayout(new FlowLayout(FlowLayout.CENTER));

	    load = new SmallButton("Load");
	    load.addActionListener(propertiesPanel);
	    add(load);

	    save = new SmallButton("Save");
	    save.addActionListener(propertiesPanel);
	    add(save);

	    def = new SmallButton("Default");
	    def.addActionListener(propertiesPanel);
	    add(def);

	    //	    add(new Label(" ")); //new Label("Spara, ladda, default"));
	    setBorder(new TitledBorder("File"));
	    //	    setBorder(new LineBorder(getPropertyBorderColor()));
	    int width = (int) getPreferredSize().getWidth();
	    int height = (int) load.getPreferredSize().getHeight();
	    Dimension d = new Dimension(width, 3 * height);
	    setPreferredSize(d);
	    setMinimumSize(d);
	    setMaximumSize(d);

	}

	public JButton getLoadButton(){
	    return load;
	}

	public JButton getSaveButton(){
	    return save;
	}

	public JButton getDefaultButton(){
	    return def;
	}

    }


    private class InfoPanel extends JPanel{
	public Property property;
	public InfoPanel(){
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setBorder(new TitledBorder("Description"));
	    putAll();
	}

	private void putAll(){
	    removeAll();

	    JPanel infoPanel = new JPanel();
	    
	    if(property==null)
		infoPanel.add(new JLabel("Press property buttons for info.")); //new Label("Info"));
	    else{
		Vector valueDescriptions = property.getAllValueDescriptions();

		/*		
		int numberOfLabels = 2;
		if(valueDescriptions!=null)
		    numberOfLabels += valueDescriptions.size()+1;
		infoPanel.setLayout(new GridLayout(numberOfLabels,1));
		*/
		infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.Y_AXIS));

		/*		
		JLabel title = new JLabel(property.getName());
		infoPanel.add(title);
		*/
		LeftLabel title = new LeftLabel(property.getName());
		infoPanel.add(title);

		/*		
		JLabel description = new JLabel(property.getDescription());
		infoPanel.add(description);
		*/

		LeftLabel description = new LeftLabel(property.getDescription());
		infoPanel.add(description);
		

		
		if(valueDescriptions!=null){
		    //		    infoPanel.add(new JLabel("Possible values:"));
		    infoPanel.add(new LeftLabel("Possible values:"));

		    /*
		    JPanel valueDescriptionPanel = new JPanel();
		    valueDescriptionPanel.setLayout(new GridLayout(valueDescriptions.size(),1));
		    */
		    for(int i=0; i<valueDescriptions.size(); i++)
			infoPanel.add(new PropertyValueDescriptionPanel((PropertyValueDescription) valueDescriptions.elementAt(i)));
		    //		    add(valueDescriptionPanel);
		}
	    }

	    JScrollPane sp = new JScrollPane();
	    sp.setViewportView(infoPanel);
	    sp.setBorder(new LineBorder(getPropertyBorderColor()));
	    add(sp);

	    int width = (int) getPreferredSize().getWidth();
	    int height = (int) (new JLabel("Dummy")).getPreferredSize().getHeight();
	    if(property==null)
		height = 4 * height;
	    else{
		Vector valueDescriptions = property.getAllValueDescriptions();
		if(valueDescriptions!=null)
		    height = 8 * height;
		else
		    height = 5 * height;
		/*
		Vector valueDescriptions = property.getAllValueDescriptions();
		int numberOfLabels = 2;
		if(valueDescriptions!=null){
		    numberOfLabels += valueDescriptions.size()+1;
		    height = (numberOfLabels+4) * height;
		}else
		    height = (numberOfLabels+4) * height;
		*/  
	    }


	    //	    d = new Dimension(PropertiesFrame.PROPERTIESFRAME_PREFERRED_SIZE.width, (int) (3*d.getHeight()));
	    Dimension d = new Dimension(width, height);
	    setPreferredSize(d);
	    setMinimumSize(d);


	    revalidate();
	    repaint();
	}

	public void setProperty(Property property){
	    this.property=property;
	    putAll();
	}

	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}
    }

    private class LeftLabel extends JPanel{
	public LeftLabel(String s){
	    setLayout(new FlowLayout(FlowLayout.LEFT));
	    JLabel l = new JLabel(s);
	    add(l);
	    //	    setPreferredSize(l.getPreferredSize());
	    JTextField dummy = new JTextField(s);
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) l.getPreferredSize().getWidth();
	    Dimension size = new Dimension(width, height);
	    setPreferredSize(size);
	}
    }


    private class PropertyGroupPanel extends JPanel{
	private PropertiesPanel propertiesPanel;
	private PropertyGroup pg;

	private PropertyPropertyGroupSorter propertyPropertyGroupSorter
	    = new PropertyPropertyGroupSorter();


	public PropertyGroupPanel(PropertyGroup pg, 
				  PropertiesPanel propertiesPanel){
	    this.pg=pg;
	    this.propertiesPanel=propertiesPanel;

	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    putAll();
	}
	    
	private void putAll(){
	    removeAll();
	    String name = pg.getPropertyGroupName();

	    JPanel namePanel = new JPanel();
	    namePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
 	    JLabel label = new JLabel("*** "+name);
	    namePanel.add(label);
	    add(namePanel);

	    Font f = label.getFont();
	    label.setFont(f.deriveFont(f.getStyle() ^ Font.BOLD));
	    
	    /*
 	    JLabel label = new JLabel("*** "+name, JLabel.LEFT);
	    add(label);
	    */

	    Vector<Property> props = pg.getPropertyVector();
	    props = propertyPropertyGroupSorter.sort(props);
	    for(int i=0; i<props.size(); i++){
		Property p = props.elementAt(i);
		if(p instanceof PropertyGroup)
		    add(new PropertyGroupPanel((PropertyGroup) p, propertiesPanel));
		else
		    add(new PropertyPanel(p, propertiesPanel));
	    }

	    revalidate();
	    repaint();
	}


	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}
    }

    private class PropertyPanel extends JPanel
	implements ActionListener{

	private Property property;
	private JButton nameButton;
	private JTextField newValueField;
	private JLabel valueLabel;

	private PropertiesPanel propertiesPanel;
		
	public PropertyPanel(Property property, 
			     PropertiesPanel propertiesPanel){
	    this.property=property;
	    this.propertiesPanel=propertiesPanel;
	    int numberOfParts = 3;
	    Vector valueDescriptions = property.getAllValueDescriptions();
	    if(valueDescriptions!=null)
		numberOfParts += 1; //valueDescriptions.size();
			
	    //setLayout(new GridLayout(numberOfParts, 1));
	    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	    setBorder(new LineBorder(propertiesPanel.getPropertyBorderColor()));

	    nameButton = new SmallButton(property.getName());
	    nameButton.addActionListener(propertiesPanel);
	    propertiesPanel.associate(nameButton, property);

	    putAll();
	}
	
	private void putAll(){
	    removeAll();
	    
	    JPanel namePanel = new JPanel();
	    namePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    namePanel.add(nameButton);
	    add(namePanel);


	    JPanel valuePanel = new JPanel();
	    valuePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
	    String value = (String) property.getValue().toString();
	    valueLabel = new JLabel("Value: "+value+", ");
	    valuePanel.add(valueLabel);

	    JPanel n = new JPanel();
	    n.setLayout(new BoxLayout(n, BoxLayout.X_AXIS));
	    JLabel nl = new JLabel("new: ");
	    n.add(nl);
	    /*
	      String newValue = "";
	      for(int i=0; i<value.length(); i++)
	      newValue += "     ";
	    */
	    newValueField = new JTextField(7); //newValue);
	    //			Dimension d = new Dimension(newValueField.getSize().width, nl.getSize().height);

	    Dimension d = newValueField.getPreferredSize();
	    newValueField.setPreferredSize(d);
	    newValueField.setMaximumSize(d);
	    newValueField.setMinimumSize(d);

	    newValueField.addActionListener(this);

	    n.add(newValueField);

	    valuePanel.add(n);

	    add(valuePanel);

	    /*
	    int width = (int) getPreferredSize().getWidth();
	    //	    int height = (int) (nameButton.getSize().getHeight() +
	    //				newValueField.getPreferredSize().getHeight());
	    int height = (int) (2.5*d.getHeight());

	    //	    d = new Dimension(PropertiesFrame.PROPERTIESFRAME_PREFERRED_SIZE.width, (int) (3*d.getHeight()));
	    d = new Dimension(width, height);
	    setPreferredSize(d);
	    //setMinimumSize(d);
	    //setMaximumSize(d);
	    */
	
	    revalidate();
	    repaint();
	}

	public void paintcomponent(Graphics g){
	    super.paintComponent(g);
	}
		
	public void actionPerformed(ActionEvent e) {
	    if(e.getSource() instanceof JTextField){    	    
		String newText = newValueField.getText();
		//System.out.println("-------------Change: "+newText);
		boolean ok=true;
				
		Vector valueDescriptions = property.getAllValueDescriptions();
		if(valueDescriptions!=null){
		    ok=false;
		    for(int i=0; i<valueDescriptions.size(); i++){
			PropertyValueDescription pvd =(PropertyValueDescription) valueDescriptions.elementAt(i);
			if(newText.equals(pvd.getValue()))
			    ok=true;
			if(property.getKind()==1){ // boolean
			    if(newText.equals("true") ||newText.equals("false"))
				ok=true;
			}
		    }
		}
				
		if(ok){
		    //System.out.println("OK!");
		    int kind = property.getKind();
		    //System.out.println("kind: "+kind);
		    if(kind==0){
			property.setValue(newText);
		    }else if(kind==1){
			boolean b = Boolean.parseBoolean(newText);
			property.setValue(b);
		    }else if(kind==2){
			try{
			    int i = Integer.parseInt(newText);
			    property.setValue(i);
			}catch(NumberFormatException nfe){
			    // Do nothing
			}
		    }else if(kind==3){
			try{
			    double d = Double.parseDouble(newText);
			    property.setValue(d);
			}catch(NumberFormatException nfe){
			    // Do nothing
			}
		    }
					
		    putAll();
		}
	    }
	}
    }

    private class PropertyValueDescriptionPanel extends JPanel{
		
	public PropertyValueDescriptionPanel(PropertyValueDescription pvd){
	    //	    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	    setLayout(new FlowLayout(FlowLayout.LEFT));
	    Object value = pvd.getValue();
	    String string = "       * "+value + " - "+pvd.getValueDescription();
	    JLabel l = new JLabel(string);
	    /*
	    JTextField dummy = new JTextField(string);
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) dummy.getPreferredSize().getWidth();
	    Dimension size = new Dimension(width, height);
	    setPreferredSize(size);
	    */
	    

	    add(l);
	}
    }

    private class SmallButton extends JButton{

	public SmallButton(String string){
	    super(string);
	    JTextField dummy = new JTextField(string);
	    JButton dummyButton = new JButton(string);
	    int height = (int) dummy.getPreferredSize().getHeight();
	    int width = (int) dummyButton.getPreferredSize().getWidth();
	    Dimension buttonSize = new Dimension(width, height);
	    setPreferredSize(buttonSize);
	}

    }
    

	
}
